"""CalibrationClientByDetectorTest class"""

import logging
import unittest

import pytest

from .calibration_test_hash import CalibrationTestHash
from .common.config_test import CONDITION, RESOURCE_NOT_FOUND
from .modules.module_base import ModuleBase


@pytest.mark.usefixtures('client_cls')
class CalibrationClientByDetectorTest(ModuleBase, unittest.TestCase):
    def setUp(self):
        self.exist_db_05 = {
            'id': -5,
            'name': 'CALIBRATION_CONSTANT_VERSION_TEST-5_DO_NOT_DELETE',
            'begin_at': '2016-05-24T12:00:00.000+02:00',
            'file_name': 'cal.r0001.c0001.h5',
            'path_to_file': '/usr/local/cal_repo/',
            'data_set_name': 'exp001.cal001',
            'raw_data_location': None,
            'calibration_constant_id': -3,
            'physical_detector_unit_id': -1,
            'flg_deployed': True,
            'flg_good_quality': True,
            'begin_validity_at': '2016-05-24T10:30:14.000+02:00',
            'end_validity_at': '2029-06-24T00:00:00.000+02:00',
            'start_idx': None,
            'end_idx': None,
            'description': 'Created automatically via seed:seed_tests'}

        self.exist_db_06 = {
            'id': -6,
            'name': 'CALIBRATION_CONSTANT_VERSION_TEST-6_DO_NOT_DELETE',

            'begin_at': '2016-05-26T12:00:00.000+02:00',
            'file_name': 'cal.r0001.c0002.h5',
            'path_to_file': '/usr/local/cal_repo/',
            'data_set_name': 'exp001.cal001',
            'raw_data_location': None,
            'calibration_constant_id': -3,
            'physical_detector_unit_id': -1,
            'flg_deployed': True,
            'flg_good_quality': True,
            'begin_validity_at': '2016-05-25T10:30:14.000+02:00',
            'end_validity_at': '2029-06-24T00:00:00.000+02:00',
            'start_idx': None,
            'end_idx': None,
            'description': 'Created automatically via seed:seed_tests'}

    def test_inject_new_calibration_constant_version(self):
        calibration_hash_01 = CalibrationTestHash(None).valid_inject_by_det_01

        cal_dict = calibration_hash_01['karabo_h']
        report_h = cal_dict['report']
        det_cond_h = cal_dict['detector_condition']
        cc_h = cal_dict['calibration_constant']
        ccv_h = cal_dict['calibration_constant_version']
        #
        begin_at_exp = calibration_hash_01['begin_at_exp']
        begin_val_at_exp = calibration_hash_01['begin_validity_at_exp']
        end_val_at_exp = calibration_hash_01['end_validity_at_exp']

        cal_inject_dict = {
            'report': report_h,
            'detector_condition': det_cond_h,
            'calibration_constant': cc_h,
            'calibration_constant_version': ccv_h
        }

        resp = self.cal_client.inject_new_calibration_constant_version(
            cal_inject_dict
        )

        # Logging information
        logging.error('calibration_hash_01: {0}'.format(calibration_hash_01))
        logging.error('cal_inject_dict: {0}'.format(cal_inject_dict))
        logging.error('resp: {0}'.format(resp))

        assert resp['app_info'] == {}
        assert resp['success'] is True

        success_info_msg = 'calibration_constant_version created successfully'
        assert resp['info'] == success_info_msg

        # Calibration Constant Version info
        resp_ccv = resp['data']
        assert resp_ccv['name'] == ccv_h['name']
        assert resp_ccv['flg_good_quality'] is True

        assert resp_ccv['begin_validity_at'] == begin_val_at_exp
        assert resp_ccv['end_validity_at'] == end_val_at_exp
        assert resp_ccv['begin_at'] == begin_at_exp

        assert resp_ccv['flg_deployed'] is True
        assert resp_ccv['raw_data_location'] == ccv_h['raw_data_location']
        assert resp_ccv['file_name'] == ccv_h['file_name']
        assert resp_ccv['path_to_file'] == ccv_h['path_to_file']
        assert resp_ccv['description'] == ccv_h['description']
        # Special cases...
        assert resp_ccv['data_set_name'] == ccv_h['data_set_name']
        assert resp_ccv['start_idx'] == ccv_h['start_idx']
        assert resp_ccv['end_idx'] == ccv_h['end_idx']

        # Calibration Constant info
        resp_cc = resp['data']['calibration_constant']
        assert resp_cc['name'] == cc_h['name']
        assert resp_cc['flg_available'] is True
        assert resp_cc['flg_auto_approve'] is True
        assert resp_cc['detector_type_id'] == -1
        # assert resp_cc['condition_id'] == -1
        assert resp_cc['calibration_id'] == -1
        assert resp_cc['description'] == cc_h['description']

        # Physical Device info
        resp_pd = resp['data']['physical_detector_unit']
        assert resp_pd['id'] == -2

        # ccv_h['pdu_physical_name'] == ''
        assert resp_pd[
                   'physical_name'] == 'PHYSICAL_DETECTOR_UNIT-2_DO_NOT_DELETE'

        assert resp_pd['detector_id'] == -2
        assert resp_pd['karabo_da'] == ccv_h['pdu_karabo_da']
        assert resp_pd['flg_available'] is True

    def test_retrieve_calibration_constant_version(self):
        calibration_hash_01 = CalibrationTestHash(
            '2016-05-26T05:00:00.757000').valid_search_by_det_01

        cal_dict_retrieve = calibration_hash_01['karabo_h']
        det_cond_h_exp = cal_dict_retrieve['detector_condition']
        cc_h_exp = cal_dict_retrieve['calibration_constant']
        ccv_h_exp = cal_dict_retrieve['calibration_constant_version']
        #
        begin_at_exp = calibration_hash_01['begin_at_exp']
        begin_val_at_exp = calibration_hash_01['begin_validity_at_exp']
        end_val_at_exp = calibration_hash_01['end_validity_at_exp']

        # cal_name = 'CALIBRATION_TEST-2_DO_NOT_DELETE'
        cal_name = calibration_hash_01['karabo_h'][
            'calibration_constant']['calibration_name']
        # pdu_phy_name = 'PHYSICAL_DETECTOR_UNIT-1_DO_NOT_DELETE'
        detector_identifier = calibration_hash_01['karabo_h'][
            'calibration_constant_version']['detector_identifier']
        pdu_karabo_da = calibration_hash_01['karabo_h'][
            'calibration_constant_version']['pdu_karabo_da']

        retrieve_h = {
            'calibration_name': cal_name,

            'pdu_physical_name': '',  # pdu_phy_name,
            'strategy': 'detector_closest_by_time',
            'detector_identifier': detector_identifier,
            'pdu_karabo_da': pdu_karabo_da,

            'measured_at': begin_at_exp,  # [Default == now()]
            'snapshot_at': '',  # [Default == now()]
            'detector_condition': det_cond_h_exp
        }

        resp = self.cal_client.retrieve_calibration_constant_version(
            retrieve_h
        )

        # Logging information
        logging.error('calibration_hash_01: {0}'.format(calibration_hash_01))
        logging.error('retrieve_h: {0}'.format(retrieve_h))
        logging.error('resp: {0}'.format(resp))

        assert resp['success'] is True
        assert resp['app_info'] == {}

        success_info_msg = 'Got calibration_constant_version successfully'
        assert resp['info'] == success_info_msg

        # Calibration Constant Version info
        resp_ccv = resp['data']

        expected_ret = {
            'id': -6,
            'name': 'CALIBRATION_CONSTANT_VERSION_TEST-6_DO_NOT_DELETE',
            'file_name': 'cal.r0001.c0002.h5',
            'path_to_file': '/usr/local/cal_repo/',
            'data_set_name': 'exp001.cal001', 'flg_deployed': True,
            'flg_good_quality': True,
            'begin_validity_at': '2016-05-25T10:30:14.000+02:00',
            'end_validity_at': '2029-06-24T00:00:00.000+02:00',
            'begin_at': '2016-05-26T12:00:00.000+02:00',
            'start_idx': None,
            'end_idx': None,
            'raw_data_location': None,
            'description': 'Created automatically via seed:seed_tests',
            'calibration_constant': {
                'id': -3,
                'name': 'CALIBRATION_CONSTANT_TEST-3_DO_NOT_DELETE',
                'flg_auto_approve': True,
                'flg_available': True,
                'description': 'Created automatically via seed:seed_tests',
                'detector_type_id': -1,
                'calibration_id': -2,
                'condition_id': -3},
            'physical_detector_unit': {
                'id': -1,
                'physical_name': 'PHYSICAL_DETECTOR_UNIT-1_DO_NOT_DELETE',
                'detector_type_id': -1,
                'flg_available': True,
                'description': None}}

        assert resp_ccv['name'] == expected_ret['name']
        assert resp_ccv['flg_good_quality'] is True

        assert resp_ccv['begin_validity_at'] == expected_ret[
            'begin_validity_at']
        assert resp_ccv['end_validity_at'] == expected_ret['end_validity_at']
        assert resp_ccv['begin_at'] == expected_ret['begin_at']

        assert resp_ccv['flg_deployed'] is True
        assert resp_ccv['raw_data_location'] == expected_ret[
            'raw_data_location']
        assert resp_ccv['file_name'] == expected_ret['file_name']
        assert resp_ccv['path_to_file'] == expected_ret['path_to_file']
        assert resp_ccv['description'] == expected_ret['description']
        # Special cases...
        assert resp_ccv['data_set_name'] == expected_ret['data_set_name']
        assert resp_ccv['start_idx'] == expected_ret['start_idx']
        assert resp_ccv['end_idx'] == expected_ret['end_idx']

        # Calibration Constant info
        resp_cc = resp_ccv['calibration_constant']
        expected_ret_cc = expected_ret['calibration_constant']

        assert resp_cc['name'] == expected_ret_cc['name']
        assert resp_cc['flg_available'] is True
        assert resp_cc['flg_auto_approve'] is True
        assert resp_cc['detector_type_id'] == -1
        # assert resp_cc['condition_id'] == -1
        assert resp_cc['calibration_id'] == -2
        assert resp_cc['description'] == expected_ret_cc['description']

        # Physical Device info
        resp_pd = resp_ccv['physical_detector_unit']
        expected_ret_pd = expected_ret['physical_detector_unit']

        assert resp_pd['id'] == -1
        assert resp_pd['physical_name'] == expected_ret_pd['physical_name']
        assert resp_pd['flg_available'] is True

    def test_retrieve_calibration_constant_version_all_detector_pdus(self):
        calibration_hash_01 = CalibrationTestHash(
            '2016-05-26T05:00:00.757000').valid_search_by_det_01

        cal_dict_retrieve = calibration_hash_01['karabo_h']
        det_cond_h_exp = cal_dict_retrieve['detector_condition']
        cc_h_exp = cal_dict_retrieve['calibration_constant']
        ccv_h_exp = cal_dict_retrieve['calibration_constant_version']
        #
        begin_at_exp = calibration_hash_01['begin_at_exp']
        begin_val_at_exp = calibration_hash_01['begin_validity_at_exp']
        end_val_at_exp = calibration_hash_01['end_validity_at_exp']

        # cal_name = 'CALIBRATION_TEST-2_DO_NOT_DELETE'
        cal_name = calibration_hash_01['karabo_h'][
            'calibration_constant']['calibration_name']
        # pdu_phy_name = 'PHYSICAL_DETECTOR_UNIT-1_DO_NOT_DELETE'
        detector_identifier = calibration_hash_01['karabo_h'][
            'calibration_constant_version']['detector_identifier']

        retrieve_h = {
            'calibration_name': cal_name,

            'pdu_physical_name': '',  # pdu_phy_name,
            'strategy': 'detector_closest_by_time',
            'detector_identifier': detector_identifier,
            'pdu_karabo_da': '',  # Should return 1 version per each PDU

            'measured_at': begin_at_exp,  # [Default == now()]
            'snapshot_at': '',  # [Default == now()]
            'detector_condition': det_cond_h_exp
        }

        resp = self.cal_client.retrieve_calibration_constant_version(
            retrieve_h
        )

        # Logging information
        logging.debug('calibration_hash_01: {0}'.format(calibration_hash_01))
        logging.debug('retrieve_h: {0}'.format(retrieve_h))
        logging.debug('resp: {0}'.format(resp))

        assert resp['success'] is True
        assert resp['app_info'] == {}

        success_info_msg = 'Got calibration_constant_version successfully'
        assert resp['info'] == success_info_msg

        # Calibration Constant Version info
        resp_ccv = resp['data']

        expected_ret = {
            'id': -6,
            'name': 'CALIBRATION_CONSTANT_VERSION_TEST-6_DO_NOT_DELETE',
            'file_name': 'cal.r0001.c0002.h5',
            'path_to_file': '/usr/local/cal_repo/',
            'data_set_name': 'exp001.cal001', 'flg_deployed': True,
            'flg_good_quality': True,
            'begin_validity_at': '2016-05-25T10:30:14.000+02:00',
            'end_validity_at': '2029-06-24T00:00:00.000+02:00',
            'begin_at': '2016-05-26T12:00:00.000+02:00',
            'start_idx': None,
            'end_idx': None,
            'raw_data_location': None,
            'description': 'Created automatically via seed:seed_tests',
            'calibration_constant': {
                'id': -3,
                'name': 'CALIBRATION_CONSTANT_TEST-3_DO_NOT_DELETE',
                'flg_auto_approve': True,
                'flg_available': True,
                'description': 'Created automatically via seed:seed_tests',
                'detector_type_id': -1,
                'calibration_id': -2,
                'condition_id': -3},
            'physical_detector_unit': {
                'id': -1,
                'physical_name': 'PHYSICAL_DETECTOR_UNIT-1_DO_NOT_DELETE',
                'detector_type_id': -1,
                'flg_available': True,
                'description': 'Created automatically via seed:seed_tests'}}

        assert resp_ccv['name'] == expected_ret['name']
        assert resp_ccv['flg_good_quality'] is True

        assert resp_ccv['begin_validity_at'] == expected_ret[
            'begin_validity_at']
        assert resp_ccv['end_validity_at'] == expected_ret['end_validity_at']
        assert resp_ccv['begin_at'] == expected_ret['begin_at']

        assert resp_ccv['flg_deployed'] is True
        assert resp_ccv['raw_data_location'] == expected_ret[
            'raw_data_location']
        assert resp_ccv['file_name'] == expected_ret['file_name']
        assert resp_ccv['path_to_file'] == expected_ret['path_to_file']
        assert resp_ccv['description'] == expected_ret['description']
        # Special cases...
        assert resp_ccv['data_set_name'] == expected_ret['data_set_name']
        assert resp_ccv['start_idx'] == expected_ret['start_idx']
        assert resp_ccv['end_idx'] == expected_ret['end_idx']

        # Calibration Constant info
        resp_cc = resp_ccv['calibration_constant']
        expected_ret_cc = expected_ret['calibration_constant']

        assert resp_cc['name'] == expected_ret_cc['name']
        assert resp_cc['flg_available'] is True
        assert resp_cc['flg_auto_approve'] is True
        assert resp_cc['detector_type_id'] == -1
        # assert resp_cc['condition_id'] == -1
        assert resp_cc['calibration_id'] == -2
        assert resp_cc['description'] == expected_ret_cc['description']

        # Physical Device info
        resp_pd = resp_ccv['physical_detector_unit']
        expected_ret_pd = expected_ret['physical_detector_unit']

        assert resp_pd['id'] == -1
        assert resp_pd['physical_name'] == expected_ret_pd['physical_name']
        assert resp_pd['flg_available'] is True

    def test_retrieve_all_calibration_constant_versions(self):
        calibration_hash_01 = CalibrationTestHash(None).valid_search_by_det_01

        cal_dict_retrieve = calibration_hash_01['karabo_h']
        det_cond_h_exp = cal_dict_retrieve['detector_condition']
        cc_h_exp = cal_dict_retrieve['calibration_constant']
        ccv_h_exp = cal_dict_retrieve['calibration_constant_version']
        #
        begin_at_exp = calibration_hash_01['begin_at_exp']
        begin_val_at_exp = calibration_hash_01['begin_validity_at_exp']
        end_val_at_exp = calibration_hash_01['end_validity_at_exp']

        # cal_name = 'CALIBRATION_TEST-2_DO_NOT_DELETE'
        cal_name = calibration_hash_01['karabo_h'][
            'calibration_constant']['calibration_name']
        # pdu_phy_name = 'PHYSICAL_DETECTOR_UNIT-1_DO_NOT_DELETE'
        detector_identifier = calibration_hash_01['karabo_h'][
            'calibration_constant_version']['detector_identifier']
        pdu_karabo_da = calibration_hash_01['karabo_h'][
            'calibration_constant_version']['pdu_karabo_da']

        retrieve_h = {
            'calibration_name': cal_name,

            # 'pdu_physical_name': pdu_phy_name,
            'strategy': 'detector_closest_by_time',
            'detector_identifier': detector_identifier,
            'pdu_karabo_da': pdu_karabo_da,

            'measured_at': begin_at_exp,  # [Default == now()]
            'snapshot_at': '',  # [Default == now()]
            'detector_condition': det_cond_h_exp
        }

        resp = self.cal_client.retrieve_all_calibration_constant_versions(
            retrieve_h
        )

        # Logging information
        logging.error('calibration_hash_01: {0}'.format(calibration_hash_01))
        logging.error('retrieve_h: {0}'.format(retrieve_h))
        logging.error('resp: {0}'.format(resp))

        assert resp['success'] is True
        assert resp['app_info'] == {}

        success_info_msg = 'Got calibration_constant_version successfully'
        assert resp['info'] == success_info_msg

        response_data = resp['data']
        index = 0
        for ccv in response_data:
            logging.error("response.data[{0}]['id'] == {1}".format(index,
                                                                   ccv['id']))
            index += 1

        # Cannot test this because new data is being added on the other tests!
        # assert len(response_data) == 2

        # Calibration Constant Version info (position 0)
        resp_ccv_0 = response_data[0]

        assert resp_ccv_0['name'] == self.exist_db_06['name']
        assert resp_ccv_0['flg_good_quality'] == self.exist_db_06[
            'flg_good_quality']

        assert resp_ccv_0['begin_validity_at'] == self.exist_db_06[
            'begin_validity_at']
        assert resp_ccv_0['end_validity_at'] == self.exist_db_06[
            'end_validity_at']
        assert resp_ccv_0['begin_at'] == self.exist_db_06['begin_at']

        assert resp_ccv_0['flg_deployed'] == self.exist_db_06['flg_deployed']
        assert resp_ccv_0['raw_data_location'] == self.exist_db_06[
            'raw_data_location']
        assert resp_ccv_0['file_name'] == self.exist_db_06['file_name']
        assert resp_ccv_0['path_to_file'] == self.exist_db_06['path_to_file']
        assert resp_ccv_0['description'] == self.exist_db_06['description']
        # Special cases...
        assert resp_ccv_0['data_set_name'] == self.exist_db_06[
            'data_set_name']
        assert resp_ccv_0['start_idx'] == self.exist_db_06['start_idx']
        assert resp_ccv_0['end_idx'] == self.exist_db_06['end_idx']

        # Calibration Constant info
        assert resp_ccv_0['calibration_constant']['id'] == self.exist_db_06[
            'calibration_constant_id']

        # Physical Device info
        assert resp_ccv_0['physical_detector_unit']['id'] == \
               self.exist_db_06['physical_detector_unit_id']

        #
        # Calibration Constant Version info (position 1)
        resp_ccv_1 = response_data[1]

        assert resp_ccv_1['name'] == self.exist_db_05['name']
        assert resp_ccv_1['flg_good_quality'] == self.exist_db_05[
            'flg_good_quality']

        assert resp_ccv_1['begin_validity_at'] == self.exist_db_05[
            'begin_validity_at']
        assert resp_ccv_1['end_validity_at'] == self.exist_db_05[
            'end_validity_at']
        assert resp_ccv_1['begin_at'] == self.exist_db_05['begin_at']

        assert resp_ccv_1['flg_deployed'] == self.exist_db_05['flg_deployed']
        assert resp_ccv_1['raw_data_location'] == self.exist_db_05[
            'raw_data_location']
        assert resp_ccv_1['file_name'] == self.exist_db_05['file_name']
        assert resp_ccv_1['path_to_file'] == self.exist_db_05['path_to_file']
        assert resp_ccv_1['description'] == self.exist_db_05['description']
        # Special cases...
        assert resp_ccv_1['data_set_name'] == self.exist_db_05[
            'data_set_name']
        assert resp_ccv_1['start_idx'] == self.exist_db_05['start_idx']
        assert resp_ccv_1['end_idx'] == self.exist_db_05['end_idx']

        # Calibration Constant info
        assert resp_ccv_1['calibration_constant']['id'] == self.exist_db_05[
            'calibration_constant_id']

        # Physical Device info
        assert resp_ccv_1['physical_detector_unit']['id'] == \
               self.exist_db_05['physical_detector_unit_id']

    def test_retrieve_ccv_no_data_found(self):
        calibration_hash_01 = CalibrationTestHash(None).valid_search_by_det_01

        cal_dict_retrieve = calibration_hash_01['karabo_h']
        det_cond_h_exp = cal_dict_retrieve['detector_condition']
        cc_h_exp = cal_dict_retrieve['calibration_constant']
        ccv_h_exp = cal_dict_retrieve['calibration_constant_version']
        #
        begin_at_exp = calibration_hash_01['begin_at_exp']
        begin_val_at_exp = calibration_hash_01['begin_validity_at_exp']
        end_val_at_exp = calibration_hash_01['end_validity_at_exp']

        # cal_name = 'CALIBRATION_TEST-2_DO_NOT_DELETE'
        cal_name = calibration_hash_01['karabo_h'][
            'calibration_constant']['calibration_name']
        # pdu_phy_name = 'PHYSICAL_DETECTOR_UNIT-3_DO_NOT_DELETE'
        detector_identifier = 'TEST_DET_CI-2'
        pdu_karabo_da = 'TEST_DAQ_DA_03'
        # pdu_phy_name = calibration_hash_01['karabo_h'][
        #     'calibration_constant_version']['pdu_physical_name']

        retrieve_h = {
            'calibration_name': cal_name,

            # 'pdu_physical_name': pdu_phy_name,
            'strategy': 'detector_closest_by_time',
            'detector_identifier': detector_identifier,
            'pdu_karabo_da': pdu_karabo_da,

            # ==================>>>
            # ERROR IS HERE:
            # Because end_validity_at was set to 1 second after the begin_at
            # and when this test is run, more than 1 second has passed
            'measured_at': '',  # [Default == now()]
            # <<<==================
            'snapshot_at': '',  # [Default == now()]
            'detector_condition': det_cond_h_exp
        }

        resp = self.cal_client.retrieve_calibration_constant_version(
            retrieve_h)

        # Logging information
        logging.error('calibration_hash_01: {0}'.format(calibration_hash_01))
        logging.error('retrieve_h: {0}'.format(retrieve_h))
        logging.error('resp: {0}'.format(resp))

        assert resp['success'] is False
        assert resp['app_info'] == RESOURCE_NOT_FOUND
        assert resp['data'] == {}

        extra_info = 'calibration_constant_version not found!'
        assert resp['info'] == extra_info

    def test_create_condition_from_dict_success(self):
        __parameters_01 = [
            {
                # 'parameter_id': '-1',
                'parameter_name': 'PARAMETER_TEST-1_DO_NOT_DELETE',
                'value': '2.5',
                'lower_deviation_value': '0.5',
                'upper_deviation_value': '0.5',
                'flg_available': 'true',
                'description': 'Created automatically via seed:seed_tests'
            },
            {
                # 'parameter_id': '-2',
                'parameter_name': 'PARAMETER_TEST-2_DO_NOT_DELETE',
                'value': '1.5',
                'lower_deviation_value': '0.5',
                'upper_deviation_value': '0.5',
                'flg_available': 'true',
                'description': 'Created automatically via seed:seed_tests'
            }
        ]

        __unique_name = 'CONDITION_TEST-1_DO_NOT_DELETE'

        cond_dict = {
            'name': __unique_name,
            'flg_available': 'true',
            'description': 'Created automatically via seed:seed_tests',
            'parameters': __parameters_01
        }

        #
        # Set Condition from DICT (should succeed with existent)
        #
        resp = self.cal_client.set_condition_from_dict(cond_dict)

        expect = {'id': -1,
                  'flg_available': True,
                  'name': 'CONDITION_TEST-1_DO_NOT_DELETE',
                  'description': 'Created automatically via seed:seed_tests',
                  'num_parameters': 2}

        msg = 'Got {0} successfully'.format(CONDITION)
        expect = {'success': True, 'info': msg, 'app_info': {}, 'data': expect}

        assert resp['app_info'] == {}
        assert resp['info'] == expect['info']
        assert resp['success'] == expect['success']

        # Validate all 'data' fields
        expect_data = expect['data']
        if type(expect_data) is dict and len(expect_data) > 0:
            for key, val in expect_data.items():
                assert resp['data'][key] == expect_data[key]
        else:
            assert resp['data'] == expect_data

    def test_create_condition_from_dict_error(self):
        __parameters_01 = [
            {
                # 'parameter_id': '-1',
                'parameter_name': 'PARAMETER_TEST-1_DO_NOT_DELETE',
                'value': '100.5',
                'lower_deviation_value': '10.5',
                'upper_deviation_value': '10.5',
                'flg_available': 'true',
                'description': 'Created automatically via seed:seed_tests'
            }
        ]

        __unique_name = 'CONDITION_TEST-1_DO_NOT_DELETE'

        expect = {
            'name': __unique_name,
            'flg_available': 'true',
            'description': 'Created automatically via seed:seed_tests',
            'parameters': __parameters_01
        }

        #
        # Set Condition from DICT (should succeed with existent)
        #
        resp = self.cal_client.set_condition_from_dict(expect)

        expect_app_info = {'name': ['has already been taken']}
        self.assert_find_error(CONDITION, resp, expect_app_info)

    def test_update_calibration_constant_version(self):

        # should succeed
        update_h = {
            'ccv_id': -1,
            'flg_good_quality': True,
            'description': 'bla',
            'end_idx': 0,
            'start_idx': 0,
            'flg_deployed': False
        }

        resp = self.cal_client.update_calibration_constant_version(update_h)

        # Logging information
        logging.error('update_h: {0}'.format(update_h))
        logging.error('resp: {0}'.format(resp))

        assert resp['success'] is True
        assert resp['app_info'] == {}

        success_info_msg = 'calibration_constant_version updated successfully'
        assert resp['info'] == success_info_msg

        # Calibration Constant Version info
        resp_ccv = resp['data']
        assert resp_ccv['flg_good_quality'] == update_h['flg_good_quality']
        assert resp_ccv['description'] == update_h['description']
        assert resp_ccv['end_idx'] == update_h['end_idx']
        assert resp_ccv['start_idx'] == update_h['start_idx']
        assert resp_ccv['flg_deployed'] == update_h['flg_deployed']

        # Update information back
        update_back_h = {
            'ccv_id': -1,
            'flg_good_quality': True,
            'description': None,
            'end_idx': None,
            'start_idx': None,
            'flg_deployed': True
        }

        resp = self.cal_client.update_calibration_constant_version(
            update_back_h
        )

        assert resp['success'] is True
        assert resp['app_info'] == {}

        success_info_msg = 'calibration_constant_version updated successfully'
        assert resp['info'] == success_info_msg

        # Calibration Constant Version info
        resp_ccv = resp['data']
        assert resp_ccv['description'] == update_back_h['description']
        assert resp_ccv['end_idx'] == update_back_h['end_idx']
        assert resp_ccv['start_idx'] == update_back_h['start_idx']
        assert resp_ccv['flg_deployed'] == update_back_h['flg_deployed']

        # should fail
        update_h_failed = {'ccv_id': -100}
        resp_failed = self.cal_client.update_calibration_constant_version(
            update_h_failed
        )

        # Logging information
        logging.error('update_h_failed: {0}'.format(update_h_failed))
        logging.error('resp_failed: {0}'.format(resp_failed))

        assert resp_failed['success'] is False
        assert resp_failed['app_info'] == 'Resource not found!'

        success_info_msg = 'calibration_constant_version not found!'
        assert resp_failed['info'] == success_info_msg

    def test_retrieve_all_phy_det_units_from_detector(self):
        calibration_hash_01 = CalibrationTestHash(
            '2016-05-26T05:00:00.757000').valid_search_by_det_01

        cal_dict_retrieve = calibration_hash_01['karabo_h']
        det_cond_h_exp = cal_dict_retrieve['detector_condition']
        cc_h_exp = cal_dict_retrieve['calibration_constant']
        ccv_h_exp = cal_dict_retrieve['calibration_constant_version']
        #
        begin_at_exp = calibration_hash_01['begin_at_exp']
        begin_val_at_exp = calibration_hash_01['begin_validity_at_exp']
        end_val_at_exp = calibration_hash_01['end_validity_at_exp']

        # cal_name = 'CALIBRATION_TEST-2_DO_NOT_DELETE'
        cal_name = calibration_hash_01['karabo_h'][
            'calibration_constant']['calibration_name']
        # pdu_phy_name = 'PHYSICAL_DETECTOR_UNIT-1_DO_NOT_DELETE'
        detector_identifier = calibration_hash_01['karabo_h'][
            'calibration_constant_version']['detector_identifier']

        retrieve_h = {
            'detector_identifier': detector_identifier,
            'snapshot_at': '',  # [Default == now()]
        }

        resp = self.cal_client.get_all_phy_det_units_from_detector(
            retrieve_h
        )

        # Logging information
        logging.debug('calibration_hash_01: {0}'.format(calibration_hash_01))
        logging.debug('retrieve_h: {0}'.format(retrieve_h))
        logging.debug('resp: {0}'.format(resp))

        assert resp['success'] is True
        assert resp['app_info'] == {}

        success_info_msg = 'Got physical_detector_unit successfully'
        assert resp['info'] == success_info_msg

        # Calibration Constant Version info
        resp_pdus = resp['data']

        exp_pdu_pos1 = {
            'id': -3,
            'physical_name': 'PHYSICAL_DETECTOR_UNIT-3_DO_NOT_DELETE',
            'detector_type_id': -1, 'detector_id': -2,
            'karabo_da': 'TEST_DAQ_DA_03'}

        assert resp_pdus[2]['id'] == exp_pdu_pos1['id']
        assert resp_pdus[2]['physical_name'] == exp_pdu_pos1['physical_name']
        assert resp_pdus[2]['detector_type_id'] == exp_pdu_pos1[
            'detector_type_id']
        assert resp_pdus[2]['detector_id'] == exp_pdu_pos1['detector_id']
        assert resp_pdus[2]['karabo_da'] == exp_pdu_pos1['karabo_da']

        exp_pdu_pos2 = {
            'id': -2,
            'physical_name': 'PHYSICAL_DETECTOR_UNIT-2_DO_NOT_DELETE',
            'detector_type_id': -1, 'detector_id': -2,
            'karabo_da': 'TEST_DAQ_DA_02'}

        assert resp_pdus[1]['id'] == exp_pdu_pos2['id']
        assert resp_pdus[1]['physical_name'] == exp_pdu_pos2['physical_name']
        assert resp_pdus[1]['detector_type_id'] == exp_pdu_pos2[
            'detector_type_id']
        assert resp_pdus[1]['detector_id'] == exp_pdu_pos2['detector_id']
        assert resp_pdus[1]['karabo_da'] == exp_pdu_pos2['karabo_da']

        exp_pdu_pos3 = {
            'id': -1,
            'physical_name': 'PHYSICAL_DETECTOR_UNIT-1_DO_NOT_DELETE',
            'detector_type_id': -1, 'detector_id': -2,
            'karabo_da': 'TEST_DAQ_DA_01'}

        assert resp_pdus[0]['id'] == exp_pdu_pos3['id']
        assert resp_pdus[0]['physical_name'] == exp_pdu_pos3['physical_name']
        assert resp_pdus[0]['detector_type_id'] == exp_pdu_pos3[
            'detector_type_id']
        assert resp_pdus[0]['detector_id'] == exp_pdu_pos3['detector_id']
        assert resp_pdus[0]['karabo_da'] == exp_pdu_pos3['karabo_da']


if __name__ == '__main__':
    unittest.main()



"""PhysicalDetectorUnitApi module class"""

import json

from ..common.base import Base
from ..common.config import *


class PhysicalDetectorUnitApi(Base):
    def create_physical_detector_unit_api(self, physical_detector_unit):
        api_url = self.__get_api_url()
        return self.api_post(api_url, data=json.dumps(physical_detector_unit))

    def delete_physical_detector_unit_api(self, physical_detector_unit_id):
        api_url = self.__get_api_url(physical_detector_unit_id)
        return self.api_delete(api_url)

    def update_physical_detector_unit_api(self, physical_detector_unit_id,
                                          physical_detector_unit):
        api_url = self.__get_api_url(physical_detector_unit_id)
        return self.api_put(api_url, data=json.dumps(physical_detector_unit))

    def get_physical_detector_unit_by_id_api(self, physical_detector_unit_id):
        api_url = self.__get_api_url(physical_detector_unit_id)
        return self.api_get(api_url, params={})

    def get_all_physical_detector_units_by_physical_name_api(self, name,
                                                             page=DEF_PAGE,
                                                             page_size=DEF_PAGE_SIZE):
        api_url = self.__get_api_url()
        return self.api_get(api_url, params={'physical_name': name,
                                             'page': page,
                                             'page_size': page_size})

    def get_all_physical_detector_units_by_detector_id_api(self, detector_id,
                                                           page=DEF_PAGE,
                                                           page_size=DEF_PAGE_SIZE):
        api_url = self.__get_api_url()
        return self.api_get(api_url, params={'detector_id': detector_id,
                                             'page': page,
                                             'page_size': page_size})

    def get_all_physical_detector_units_by_det_and_krbda_api(self,
                                                             detector_id,
                                                             karabo_da,
                                                             page=DEF_PAGE,
                                                             page_size=DEF_PAGE_SIZE):
        api_url = self.__get_api_url()
        return self.api_get(api_url, params={'detector_id': detector_id,
                                             'karabo_da': karabo_da,
                                             'page': page,
                                             'page_size': page_size})

    #
    def get_all_by_detector_api(self, detector_id, snapshot_at):
        api_action_name = '/get_all_by_detector'
        api_url = self.__get_api_url(api_action_name)

        params = {'detector_id': str(detector_id),
                  'snapshot_at': snapshot_at}

        return self.api_get(api_url, params=params)

    #
    # Private Methods
    #
    def __get_api_url(self, api_specifics=''):
        model_name = 'physical_detector_units/'
        return self.get_api_url(model_name, api_specifics)



"""DetectorTest class"""

import unittest

import pytest

from .module_base import ModuleBase
from ..common.config_test import DETECTOR, RESOURCE_NOT_FOUND
from ...modules.detector import Detector

MODULE_NAME = DETECTOR


@pytest.mark.usefixtures('client_cls')
class DetectorTest(ModuleBase, unittest.TestCase):
    def setUp(self):
        __unique_name1 = 'Detector01-Dynamic-TO_DELETED'
        self.detector_1 = {
            'name': __unique_name1,
            'identifier': __unique_name1,
            'karabo_name': __unique_name1,
            'karabo_id_control': 'Q1M1',
            'flg_available': True,
            'description': 'desc 01'
        }

        __unique_name_upd = 'Det01-Dynamic_UPD-TO_DELETED'
        self.detector_1_upd = {
            'name': __unique_name_upd,
            'identifier': __unique_name_upd,
            'karabo_name': __unique_name_upd,
            'karabo_id_control': 'Q1M2',
            'flg_available': False,
            'description': 'desc 01 Updated!'
        }

    def test_create_detector(self):

        print('*' * 100)
        print('detector_01[name] =>', str(self.detector_1['name']))
        print('detector_01[identifier] =>', str(self.detector_1['identifier']))
        print('detector_01[karabo_name] =>',
              str(self.detector_1['karabo_name']))
        print('detector_01[karabo_id_control] =>',
              str(self.detector_1['karabo_id_control']))
        print('*' * 100)

        detector_01 = Detector(
            calibration_client=self.cal_client,
            name=self.detector_1['name'],
            identifier=self.detector_1['identifier'],
            karabo_name=self.detector_1['karabo_name'],
            karabo_id_control=self.detector_1['karabo_id_control'],
            flg_available=self.detector_1['flg_available'],
            description=self.detector_1['description'])

        #
        # Create new entry (should succeed)
        #
        result1 = detector_01.create()

        try:
            self.assert_create_success(MODULE_NAME, result1, self.detector_1)

            detector_id = result1['data']['id']
            detector_name = result1['data']['name']

            # Create duplicated entry (should throw an error)
            detector_01_dup = detector_01
            result2 = detector_01_dup.create()
            expect_app_info = {'name': ['has already been taken'],
                               'identifier': ['has already been taken'],
                               'karabo_name': ['has already been taken']}
            self.assert_create_error(MODULE_NAME, result2, expect_app_info)

            # Get entry by name
            result3 = Detector.get_by_name(self.cal_client, detector_name)
            self.assert_find_success(MODULE_NAME, result3, self.detector_1)

            # Get entry by ID
            result4 = Detector.get_by_id(self.cal_client, detector_id)
            self.assert_find_success(MODULE_NAME, result4, self.detector_1)

            # Get entry with non-existent ID (should throw an error)
            result5 = Detector.get_by_id(self.cal_client, -666)
            self.assert_find_error(MODULE_NAME, result5, RESOURCE_NOT_FOUND)

            # Put entry information (update some fields should succeed)
            detector_01.name = self.detector_1_upd['name']
            detector_01.flg_available = self.detector_1_upd['flg_available']
            detector_01.description = self.detector_1_upd['description']
            result6 = detector_01.update()
            self.assert_update_success(MODULE_NAME, result6,
                                       self.detector_1_upd)

            # Put entry information (update some fields should throw an error)
            detector_01.name = '__THIS_NAME_IS_1_CHARACTERS_LONGER_THAN_THE_ALLOWED_MAX_NUM_1234__'  # noqa
            detector_01.flg_available = self.detector_1_upd['flg_available']
            detector_01.description = self.detector_1_upd['description']
            result7 = detector_01.update()
            expect_app_info = {
                'name': ['is too long (maximum is 64 characters)']}
            self.assert_update_error(MODULE_NAME, result7, expect_app_info)

        finally:
            # Delete entry (should succeed)
            # (test purposes only to keep the DB clean)
            result8 = detector_01.delete()

        # Check deletion worked if there isn't already an error to report
        self.assert_delete_success(MODULE_NAME, result8)

        # Delete entry (should throw an error)
        # (test purposes only to keep the DB clean)
        result9 = detector_01.delete()
        self.assert_delete_error(MODULE_NAME, result9, RESOURCE_NOT_FOUND)

    def fields_validation(self, receive, expect):
        assert receive['name'] == expect['name']
        assert receive['flg_available'] == expect['flg_available']
        assert receive['description'] == expect['description']


if __name__ == '__main__':
    unittest.main()



"""Calibration module class"""

from http import HTTPStatus

from ..common.base import Base
from ..common.config import *

MODULE_NAME = CALIBRATION


class Calibration:
    def __init__(self, calibration_client,
                 name, unit_id,
                 max_value, min_value,
                 allowed_deviation, description=''):
        self.calibration_client = calibration_client
        self.id = None
        self.name = name
        self.unit_id = unit_id
        self.max_value = max_value
        self.min_value = min_value
        self.allowed_deviation = allowed_deviation
        self.description = description

    def create(self):
        cal_client = self.calibration_client
        response = cal_client.create_calibration_api(self.__get_resource())

        Base.cal_debug(MODULE_NAME, CREATE, response)
        res = Base.format_response(
            response, CREATE, HTTPStatus.CREATED, MODULE_NAME
        )

        if res['success']:
            self.id = res['data']['id']

        return res

    def delete(self):
        cal_client = self.calibration_client
        response = cal_client.delete_calibration_api(self.id)

        Base.cal_debug(MODULE_NAME, DELETE, response)
        return Base.format_response(
            response, DELETE, HTTPStatus.NO_CONTENT, MODULE_NAME
        )

    def update(self):
        cal_client = self.calibration_client
        response = cal_client.update_calibration_api(self.id,
                                                     self.__get_resource())

        Base.cal_debug(MODULE_NAME, UPDATE, response)
        return Base.format_response(
            response, UPDATE, HTTPStatus.OK, MODULE_NAME
        )

    @staticmethod
    def get_by_id(cal_client, calibration_id):
        response = cal_client.get_calibration_by_id_api(calibration_id)

        Base.cal_debug(MODULE_NAME, 'get_by_id', response)
        return Base.format_response(response, GET, HTTPStatus.OK, MODULE_NAME)

    @staticmethod
    def get_all_by_name(cal_client, name, page=DEF_PAGE, page_size=DEF_PAGE_SIZE):
        response = cal_client.get_all_calibrations_by_name_api(name, page, page_size)

        Base.cal_debug(MODULE_NAME, 'get_all_by_name', response)
        return Base.format_response(response, GET, HTTPStatus.OK, MODULE_NAME)

    @staticmethod
    def get_by_name(cal_client, name):
        res = Calibration.get_all_by_name(cal_client, name)

        if res['success']:
            if res['data'] == []:
                resp_data = []
            else:
                resp_data = res['data'][0]

            res = {'success': res['success'],
                   'info': res['info'],
                   'app_info': res['app_info'],
                   'data': resp_data}

        return res

    # @staticmethod
    # def get_calibration_constant(cal_client, calibration_id,
    #                              detector_type_id, condition_id):
    #     res = cal_client.get_calibration_constant_by_uk_api(cal_client,
    #                                                         calibration_id,
    #                                                         detector_type_id,
    #                                                         condition_id)
    #
    #     Base.cal_debug(MODULE_NAME, 'get_calibration_constant', res)
    #     return Base.format_response(res, GET, OK, MODULE_NAME)

    def __get_resource(self):
        calibration = {
            CALIBRATION: {
                'name': self.name,
                'unit_id': self.unit_id,
                'max_value': self.max_value,
                'min_value': self.min_value,
                'allowed_deviation': self.allowed_deviation,
                'description': self.description
            }
        }

        return calibration



"""UserApiTest class"""

import unittest
from http import HTTPStatus

import pytest

from .api_base import ApiBase
from ..common.secrets import USER_INFO


@pytest.mark.usefixtures('client_cls')
class UserApiTest(ApiBase, unittest.TestCase):
    __current_user_info_01 = {
        'email': USER_INFO['EMAIL'],
        'first_name': USER_INFO['FIRST_NAME'],
        'last_name': USER_INFO['LAST_NAME'],
        'name': USER_INFO['NAME'],
        'nickname': USER_INFO['NICKNAME'],
        'provider': USER_INFO['PROVIDER'],
        'uid': USER_INFO['UID']
    }

    def test_user_info(self):
        current_user = self.__current_user_info_01

        resp = self.cal_client.get_current_user()
        receive = self.load_response_content(resp)

        # Debug Response
        # debug_response(response)

        self.fields_validation(receive, current_user)
        assert resp.status_code == HTTPStatus.OK

    def fields_validation(self, receive, expect):
        assert receive['email'] == expect['email']
        assert receive['first_name'] == expect['first_name']
        assert receive['last_name'] == expect['last_name']
        assert receive['name'] == expect['name']
        assert receive['nickname'] == expect['nickname']
        assert receive['provider'] == expect['provider']
        assert receive['uid'] == expect['uid']


if __name__ == '__main__':
    unittest.main()



"""ParameterApi module class"""

import json

from ..common.base import Base
from ..common.config import *


class ParameterApi(Base):
    def create_parameter_api(self, parameter):
        api_url = self.__get_api_url()
        return self.api_post(api_url, data=json.dumps(parameter))

    def delete_parameter_api(self, parameter_id):
        api_url = self.__get_api_url(parameter_id)
        return self.api_delete(api_url)

    def update_parameter_api(self, parameter_id, parameter):
        api_url = self.__get_api_url(parameter_id)
        return self.api_put(api_url, data=json.dumps(parameter))

    def get_parameter_by_id_api(self, parameter_id):
        api_url = self.__get_api_url(parameter_id)
        return self.api_get(api_url, params={})

    def get_all_parameters_by_name_api(self, name,
                                       page=DEF_PAGE,
                                       page_size=DEF_PAGE_SIZE):
        api_url = self.__get_api_url()
        return self.api_get(api_url, params={'name': name,
                                             'page': page,
                                             'page_size': page_size})

    #
    # Private Methods
    #
    def __get_api_url(self, api_specifics=''):
        model_name = 'parameters/'
        return self.get_api_url(model_name, api_specifics)



"""DetectorTypeApiTest class"""

import unittest
from http import HTTPStatus

import pytest

from .api_base import ApiBase
from ..common.generators import generate_unique_name
from ..common.config_test import *


@pytest.mark.usefixtures('client_cls')
class DetectorTypeApiTest(ApiBase, unittest.TestCase):
    def test_create_detector_type_api(self):
        __unique_name = generate_unique_name('DetectorTypeApi')
        detector_type = {
            'detector_type': {
                'name': __unique_name,
                'flg_available': True,
                'description': 'desc 01'
            }
        }

        expect = detector_type['detector_type']

        # Create new entry (should succeed)
        received = self.__create_entry_api(detector_type, expect)

        detector_type_id = received['id']
        detector_type_name = received['name']

        try:
            # Create duplicated entry (should throw an error)
            self.__create_error_entry_uk_api(detector_type)

            # Get entry by name
            self.__get_all_entries_by_name_api(detector_type_name, expect)

            # Get entry by ID
            self.__get_entry_by_id_api(detector_type_id, expect)

            # Put entry information (update some fields should succeed)
            self.__update_entry_api(detector_type_id, expect)

        finally:
            # Delete entry (should succeed)
            # (test purposes only to keep the DB clean)
            del_resp = self.cal_client.delete_detector_type_api(
                detector_type_id)

        # Validate the response from deletion if nothing already error-ed
        self.get_and_validate_delete_entry_by_id(del_resp)

    def fields_validation(self, receive, expect):
        assert receive['name'] == expect['name']
        assert receive['flg_available'] == expect['flg_available']
        assert receive['description'] == expect['description']

    #
    # Internal private APIs methods
    #
    def __create_entry_api(self, entry_info, expect):
        response = self.cal_client.create_detector_type_api(entry_info)
        receive = self.get_and_validate_create_entry(response)
        self.fields_validation(receive, expect)
        return receive

    def __create_error_entry_uk_api(self, entry_info):
        response = self.cal_client.create_detector_type_api(entry_info)
        receive = self.load_response_content(response)

        expect = {'info': {'name': ['has already been taken']}}
        assert receive == expect
        assert response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY

    def __update_entry_api(self, entry_id, expect):
        unique_name_upd = generate_unique_name('DetectorTypeApiUpd')
        detector_type_upd = {
            'detector_type': {
                'name': unique_name_upd,
                'flg_available': False,
                'description': 'desc 01 updated!!!'
            }
        }

        response = self.cal_client.update_detector_type_api(entry_id,
                                                            detector_type_upd)
        receive = self.load_response_content(response)

        expect_upd = detector_type_upd['detector_type']

        self.fields_validation(receive, expect_upd)
        assert response.status_code == HTTPStatus.OK

        assert expect['name'] != expect_upd['name']
        assert expect['flg_available'] != expect_upd['flg_available']
        assert expect['description'] != expect_upd['description']

    def __get_all_entries_by_name_api(self, name, expect,
                                      page=DEF_PAGE, page_size=DEF_PAGE_SIZE):
        response = self.cal_client.get_all_detector_types_by_name_api(name, page, page_size)
        receive = self.get_and_validate_all_entries_by_name(response)
        self.fields_validation(receive, expect)

    def __get_entry_by_id_api(self, entry_id, expect):
        response = self.cal_client.get_detector_type_by_id_api(entry_id)
        receive = self.get_and_validate_entry_by_id(response)
        self.fields_validation(receive, expect)


if __name__ == '__main__':
    unittest.main()



"""ParameterApiTest class"""

import unittest
from http import HTTPStatus

import pytest

from .api_base import ApiBase
from ..common.config_test import *
from ..common.generators import generate_unique_name


@pytest.mark.usefixtures('client_cls')
class ParameterApiTest(ApiBase, unittest.TestCase):
    def test_create_parameter_api(self):
        __unique_name = generate_unique_name('ParameterApi')
        parameter = {
            PARAMETER: {
                'name': __unique_name,
                'unit_id': -1,
                'flg_available': True,
                'flg_logarithmic': False,
                'def_lower_deviation_value': 1.0,
                'def_upper_deviation_value': 1.2,
                'description': 'desc 01'
            }
        }

        expect = parameter[PARAMETER]

        # Create new entry (should succeed)
        received = self.__create_entry_api(parameter, expect)

        parameter_id = received['id']
        parameter_name = received['name']

        try:
            # Create duplicated entry (should throw an error)
            self.__create_error_entry_uk_api(parameter)

            # Get entry by name
            self.__get_all_entries_by_name_api(parameter_name, expect)

            # Get entry by ID
            self.__get_entry_by_id_api(parameter_id, expect)

            # Put entry information (update some fields should succeed)
            self.__update_entry_api(parameter_id, expect)

        finally:
            # Delete entry (should succeed)
            # (test purposes only to keep the DB clean)
            del_resp = self.cal_client.delete_parameter_api(parameter_id)

        # Validate the response from deletion if nothing already error-ed
        self.get_and_validate_delete_entry_by_id(del_resp)

    def fields_validation(self, receive, expect):
        assert receive['name'] == expect['name']
        assert receive['unit_id'] == expect['unit_id']
        assert receive['flg_available'] == expect['flg_available']
        assert receive['flg_logarithmic'] == expect['flg_logarithmic']
        assert receive['def_lower_deviation_value'] \
               == expect['def_lower_deviation_value']
        assert receive['def_upper_deviation_value'] \
               == expect['def_upper_deviation_value']
        assert receive['description'] == expect['description']

    def __create_entry_api(self, entry_info, expect):
        response = self.cal_client.create_parameter_api(entry_info)
        receive = self.get_and_validate_create_entry(response)
        self.fields_validation(receive, expect)

        return receive

    def __create_error_entry_uk_api(self, entry_info):
        response = self.cal_client.create_parameter_api(entry_info)
        receive = self.load_response_content(response)

        expect = {'info': {'name': ['has already been taken']}}
        assert receive == expect
        assert response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY

    def __update_entry_api(self, entry_id, expect):
        unique_name_upd = generate_unique_name('ParameterApiUpd')
        parameter_upd = {
            PARAMETER: {
                'name': expect['name'],
                # 'unit_id': '-1',
                'flg_available': False,
                'flg_logarithmic': True,
                'def_lower_deviation_value': 0.0,
                'def_upper_deviation_value': 10.2,
                'description': 'desc 01 updated!!!'
            }
        }

        res = self.cal_client.update_parameter_api(entry_id, parameter_upd)
        receive = self.load_response_content(res)

        # Add parameters not send to the update API
        parameter_upd[PARAMETER]['unit_id'] = -1
        expect_upd = parameter_upd[PARAMETER]

        self.fields_validation(receive, expect_upd)
        assert res.status_code == HTTPStatus.OK

        assert expect['flg_available'] != expect_upd['flg_available']
        assert expect['flg_logarithmic'] != expect_upd['flg_logarithmic']
        assert expect['def_lower_deviation_value'] \
               != expect_upd['def_lower_deviation_value']
        assert expect['def_upper_deviation_value'] \
               != expect_upd['def_upper_deviation_value']
        assert expect['description'] != expect_upd['description']

    def __get_all_entries_by_name_api(self, name, expect,
                                      page=DEF_PAGE, page_size=DEF_PAGE_SIZE):
        response = self.cal_client.get_all_parameters_by_name_api(name, page, page_size)
        receive = self.get_and_validate_all_entries_by_name(response)
        self.fields_validation(receive, expect)

    def __get_entry_by_id_api(self, entry_id, expect):
        response = self.cal_client.get_parameter_by_id_api(entry_id)
        receive = self.get_and_validate_entry_by_id(response)
        self.fields_validation(receive, expect)


if __name__ == '__main__':
    unittest.main()



"""Generators Class with helper methods to generate dummy data"""

import logging
import socket
import time
from random import randrange


def generate_unique_name(prefix):
    dt_str = time.strftime('%Y-%m-%d %H:%M:%S', time.gmtime())
    host = socket.gethostname()
    rand = randrange(999)

    unique_str = '{0} {1} {2} {3}'.format(prefix, dt_str, rand, host)
    if len(unique_str) > 60:
        unique_str = '{0}..'.format(unique_str[:58])

    logging.debug('generate::unique_name == {0}'.format(unique_str))

    return unique_str


def generate_unique_file_name():
    epoch_str = str(time.time())  # Length == 17

    # host Max length => 60 - (17 + 8) = 35!
    host = str(socket.gethostname())[0:34]  # 0 - 34 == 35

    unique_file = "cal.{0}_{1}.h5".format(epoch_str, host)
    logging.debug('generate::unique_file_name == {0}'.format(unique_file))

    return unique_file


def generate_timestamp_str(secs=None):
    epoch_time = int(time.time())

    if secs is None:
        additional_secs = -randrange(60)
    else:
        additional_secs = secs

    ts = time.gmtime(epoch_time + additional_secs)
    dt_str = time.strftime('%Y-%m-%dT%H:%M:%S.000+00:00', ts)

    logging.debug('generate_timestamp_str == {0}'.format(dt_str))
    return dt_str



"""CalibrationApi module class"""

import json

from ..common.base import Base
from ..common.config import *


class CalibrationApi(Base):
    def create_calibration_api(self, calibration):
        api_url = self.__get_api_url()
        return self.api_post(api_url, data=json.dumps(calibration))

    def delete_calibration_api(self, calibration_id):
        api_url = self.__get_api_url(calibration_id)
        return self.api_delete(api_url)

    def update_calibration_api(self, calibration_id, calibration):
        api_url = self.__get_api_url(calibration_id)
        return self.api_put(api_url, data=json.dumps(calibration))

    def get_calibration_by_id_api(self, calibration_id):
        api_url = self.__get_api_url(calibration_id)
        return self.api_get(api_url, params={})

    def get_all_calibrations_by_name_api(self, name,
                                         page=DEF_PAGE,
                                         page_size=DEF_PAGE_SIZE):
        api_url = self.__get_api_url()
        return self.api_get(api_url, params={'name': name,
                                             'page': page,
                                             'page_size': page_size})

    #
    def get_all_calibration_constants_by_conds_api(self, calibration_id,
                                                   dev_type_id, cond_ids):
        api_action_name = '/get_all_calibration_constants'
        api_relative_url = '{0}{1}'.format(calibration_id, api_action_name)
        api_url = self.__get_api_url(api_relative_url)

        params = {'calibration_id': str(calibration_id),
                  'detector_type_id': str(dev_type_id),
                  'condition_ids': str(cond_ids)}

        return self.api_get(api_url, params=params)

    #
    def get_calibration_constant_by_uk_api(self, calibration_id,
                                           dev_type_id, cond_id):
        api_action_name = '/get_calibration_constant'
        api_relative_url = '{0}{1}'.format(calibration_id, api_action_name)
        api_url = self.__get_api_url(api_relative_url)

        params = {'calibration_id': str(calibration_id),
                  'detector_type_id': str(dev_type_id),
                  'condition_id': str(cond_id)}

        return self.api_get(api_url, params=params)

    #
    # Private Methods
    #
    def __get_api_url(self, api_specifics=''):
        model_name = 'calibrations/'
        return self.get_api_url(model_name, api_specifics)



"""CalibrationConstantVersionTest class"""

import unittest

import pytest
from dateutil.parser import parse as parse_dt

from .module_base import ModuleBase
from ..common import generators
from ..common.config_test import (
    CALIBRATION_CONSTANT_VERSION, PHYSICAL_DETECTOR_UNIT, RESOURCE_NOT_FOUND
)
from ...modules.calibration_constant_version import CalibrationConstantVersion

MODULE_NAME = CALIBRATION_CONSTANT_VERSION


@pytest.mark.usefixtures('client_cls')
class CalibrationConstantVersionTest(ModuleBase, unittest.TestCase):
    def setUp(self):
        __unique_name = generators.generate_unique_name('CCV_01')
        __unique_file_name = generators.generate_unique_file_name()
        __begin_at = generators.generate_timestamp_str(0)

        self.ccv_01 = {
            'name': __unique_name,
            'file_name': __unique_file_name,
            'path_to_file': 'path_to_file_1',
            'data_set_name': 'data_set_name_1',
            'calibration_constant_id': -3,
            'physical_detector_unit_id': -1,
            'flg_deployed': True,
            'flg_good_quality': True,
            'begin_validity_at': '2014-05-25T08:30:00.000+02:00',
            'end_validity_at': '2025-12-25T08:30:00.000+01:00',
            'begin_at': __begin_at,
            'start_idx': 1,
            'end_idx': 2,
            'raw_data_location': 'XFEL/metadata/',
            'report_id': '',
            'description': 'desc version 01'
        }

        __unique_name_upd = generators.generate_unique_name('CCVersionUpd01')
        __unique_file_name_upd = generators.generate_unique_file_name()
        __new_begin_at = -(60 * 60 * 24)  # Minus 1 day
        __begin_at_upd = generators.generate_timestamp_str(__new_begin_at)

        self.ccv_01_upd = {
            'name': __unique_name_upd,
            'file_name': __unique_file_name_upd,
            'path_to_file': 'path_to_file_1_upd',
            'data_set_name': 'data_set_name_1_upd',
            'calibration_constant_id': -3,
            'physical_detector_unit_id': -1,
            'flg_deployed': False,
            'flg_good_quality': False,
            'begin_validity_at': '2014-06-25T08:30:00.000+02:00',
            'end_validity_at': '2025-11-25T08:30:00.000+01:00',
            'begin_at': __begin_at_upd,
            'start_idx': 0,
            'end_idx': 1,
            'raw_data_location': 'XFEL/metadata/bck/',
            'report_id': '',
            'description': 'desc version 01 Updated!'
        }

        self.expected_db_01 = {
            'id': -1,
            'name': 'CALIBRATION_CONSTANT_VERSION_TEST-1_DO_NOT_DELETE',
            'file_name': 'cal.r0001.c0002.h5',
            'path_to_file': '/usr/local/cal_repo/',
            'data_set_name': 'exp001.cal001',
            'flg_deployed': True,
            'flg_good_quality': True,
            'begin_validity_at': '2014-05-20T10:30:14.000+02:00',
            'end_validity_at': '2029-06-24T00:00:00.000+02:00',
            'begin_at': '2014-05-20T12:00:00.000+02:00',
            'start_idx': None,
            'end_idx': None,
            'raw_data_location': None,
            'description': None,
            # 'calibration_constant': {
            #     'id': -1,
            #     'name': 'CALIBRATION_CONSTANT_TEST-1_DO_NOT_DELETE',
            #     'flg_auto_approve': True,
            #     'flg_available': True,
            #     'description': 'Created automatically via seed:seed_tests',
            #     'detector_type_id': -1,
            #     'calibration_id': -1,
            #     'condition_id': -1},
            'calibration_constant_id': -1,
            # 'physical_detector_unit': {
            #     'id': -1,
            #     'name': 'PHYSICAL_DETECTOR_UNIT-1_DO_NOT_DELETE',
            #     'detector_type_id': -1,
            #     'flg_available': True,
            #     'description': 'Created automatically via seed:seed_tests'}
            'physical_detector_unit_id': -1}

        self.expected_db_02 = {
            'id': -2,
            'name': 'CALIBRATION_CONSTANT_VERSION_TEST-2_DO_NOT_DELETE',
            'file_name': 'cal.r0001.c0002.h5',
            'path_to_file': '/usr/local/cal_repo/',
            'data_set_name': 'exp001.cal001',
            'flg_deployed': True,
            'flg_good_quality': True,
            'begin_validity_at': '2014-05-25T10:30:14.000+02:00',
            'end_validity_at': '2029-06-24T00:00:00.000+02:00',
            'begin_at': '2014-05-26T12:00:00.000+02:00',
            'start_idx': None,
            'end_idx': None,
            'raw_data_location': None,
            'description': 'Created automatically via seed:seed_tests',
            # 'calibration_constant': {
            #     'id': -1,
            #     'name': 'CALIBRATION_CONSTANT_TEST-1_DO_NOT_DELETE',
            #     'flg_auto_approve': True,
            #     'flg_available': True,
            #     'description': 'Created automatically via seed:seed_tests',
            #     'detector_type_id': -1,
            #     'calibration_id': -1,
            #     'condition_id': -1},
            'calibration_constant_id': -1,
            # 'physical_detector_unit': {
            #     'id': -1,
            #     'name': 'PHYSICAL_DETECTOR_UNIT-1_DO_NOT_DELETE',
            #     'detector_type_id': -1,
            #     'flg_available': True,
            #     'description': 'Created automatically via seed:seed_tests'}
            'physical_detector_unit_id': -1}

        self.expected_db_04 = {
            'id': -4,
            'name': 'CALIBRATION_CONSTANT_VERSION_TEST-4_DO_NOT_DELETE',
            'file_name': 'cal.r0001.c0002.h5',
            'path_to_file': '/usr/local/cal_repo/',
            'data_set_name': 'exp001.cal001',
            'flg_deployed': True,
            'flg_good_quality': True,
            'begin_validity_at': '2015-05-25T10:30:14.000+02:00',
            'end_validity_at': '2029-06-24T00:00:00.000+02:00',
            'begin_at': '2015-05-26T12:00:00.000+02:00',
            'start_idx': None,
            'end_idx': None,
            'raw_data_location': None,
            'description': 'Created automatically via seed:seed_tests',
            # 'calibration_constant': {
            #     'id': -2,
            #     'name': 'CALIBRATION_CONSTANT_TEST-2_DO_NOT_DELETE',
            #     'flg_auto_approve': True,
            #     'flg_available': True,
            #     'description': 'Created automatically via seed:seed_tests',
            #     'detector_type_id': -1,
            #     'calibration_id': -1,
            #     'condition_id': -2},
            'calibration_constant_id': -2,
            # 'physical_detector_unit': {
            #     'id': -1,
            #     'name': 'PHYSICAL_DETECTOR_UNIT-1_DO_NOT_DELETE',
            #     'detector_type_id': -1,
            #     'flg_available': True,
            #     'description': 'Created automatically via seed:seed_tests'}
            'physical_detector_unit_id': -1}

    def test_create_calibration_constant_version(self):
        __begin_at = generators.generate_timestamp_str(0)

        __new_begin_at = -(60 * 60 * 24)  # Minus 1 day
        __begin_at_upd = generators.generate_timestamp_str(__new_begin_at)

        ccv_01 = CalibrationConstantVersion(
            calibration_client=self.cal_client,
            name=self.ccv_01['name'],
            file_name=self.ccv_01['file_name'],
            path_to_file=self.ccv_01['path_to_file'],
            data_set_name=self.ccv_01['data_set_name'],
            calibration_constant_id=self.ccv_01['calibration_constant_id'],
            physical_detector_unit_id=self.ccv_01['physical_detector_unit_id'],
            flg_deployed=self.ccv_01['flg_deployed'],
            flg_good_quality=self.ccv_01['flg_good_quality'],
            begin_validity_at=self.ccv_01['begin_validity_at'],
            end_validity_at=self.ccv_01['end_validity_at'],
            begin_at=__begin_at,
            start_idx=self.ccv_01['start_idx'],
            end_idx=self.ccv_01['end_idx'],
            raw_data_location=self.ccv_01['raw_data_location'],
            report_id=self.ccv_01['report_id'],
            description=self.ccv_01['description']
        )

        #
        # Create new entry (should succeed)
        #
        result1 = ccv_01.create()
        self.assert_create_success(MODULE_NAME, result1, self.ccv_01)

        calibration_constant_vers = result1['data']
        ccv_id = calibration_constant_vers['id']
        ccv_name = calibration_constant_vers['name']
        ccv_cc_id = calibration_constant_vers['calibration_constant']['id']
        ccv_pd_id = calibration_constant_vers['physical_detector_unit']['id']

        #
        # Create duplicated entry (should throw an error)
        #
        ccv_01_dup = ccv_01
        result2 = ccv_01_dup.create()
        expect_app_info = {'name': ['has already been taken']}
        self.assert_create_error(MODULE_NAME, result2, expect_app_info)

        #
        # Get entry by name
        #
        result3 = CalibrationConstantVersion.get_by_name(self.cal_client,
                                                         ccv_name)
        self.assert_find_success(MODULE_NAME, result3, self.ccv_01)

        #
        # Get entry by ID
        #
        result4 = CalibrationConstantVersion.get_by_id(self.cal_client, ccv_id)
        self.assert_find_success(MODULE_NAME, result4, self.ccv_01)

        #
        # Get entry with non-existent ID (should throw an error)
        #
        ccv_id = -666
        result5 = CalibrationConstantVersion.get_by_id(self.cal_client, ccv_id)
        self.assert_find_error(MODULE_NAME, result5, RESOURCE_NOT_FOUND)

        #
        # Get entry by UK (should succeed)
        # (calibration_constant_id,
        #  physical_detector_unit_id,
        #  event_at,
        #  snapshot_at)
        #
        ccv_event_at = calibration_constant_vers['begin_at']
        ccv_snapshot_at = ''  # '' == Now()

        result_uk = CalibrationConstantVersion.get_by_uk(self.cal_client,
                                                         ccv_cc_id,
                                                         ccv_pd_id,
                                                         ccv_event_at,
                                                         ccv_snapshot_at)
        self.assert_find_success(MODULE_NAME, result_uk, self.ccv_01)

        #
        # Get entry by UK (should throw an error)
        # (calibration_constant_id,
        #  physical_detector_unit_id,
        #  event_at,
        #  snapshot_at)
        #
        calibration_constant_id = -666
        physical_detector_unit_id = -666
        ccv_event_at = ''  # '' == Now()
        ccv_snapshot_at = ''  # '' == Now()

        res_uk_error = CalibrationConstantVersion.get_by_uk(
            self.cal_client,
            calibration_constant_id,
            physical_detector_unit_id,
            ccv_event_at,
            ccv_snapshot_at
        )
        self.assert_find_error(MODULE_NAME, res_uk_error, RESOURCE_NOT_FOUND)

        #
        # Put entry information (update some fields should succeed)
        #
        ccv_01.name = self.ccv_01_upd['name']
        ccv_01.file_name = self.ccv_01_upd['file_name']
        ccv_01.path_to_file = self.ccv_01_upd['path_to_file']
        ccv_01.data_set_name = self.ccv_01_upd['data_set_name']
        ccv_01.flg_deployed = self.ccv_01_upd['flg_deployed']
        ccv_01.flg_good_quality = self.ccv_01_upd['flg_good_quality']
        ccv_01.begin_validity_at = self.ccv_01_upd['begin_validity_at']
        ccv_01.end_validity_at = self.ccv_01_upd['end_validity_at']
        ccv_01.begin_at = __begin_at_upd
        ccv_01.start_idx = self.ccv_01_upd['start_idx']
        ccv_01.end_idx = self.ccv_01_upd['end_idx']
        ccv_01.raw_data_location = self.ccv_01_upd['raw_data_location']
        ccv_01.description = self.ccv_01_upd['description']
        result6 = ccv_01.update()
        self.assert_update_success(MODULE_NAME, result6, self.ccv_01_upd)

        #
        # Put entry information (update some fields should throw an error)
        #
        ccv_01.name = '__THIS_NAME_IS_1_CHARACTERS_LONGER_THAN_THE_ALLOWED_MAX_NUM__'  # noqa
        result7 = ccv_01.update()
        expect_app_info = {'name': ['is too long (maximum is 60 characters)']}
        self.assert_update_error(MODULE_NAME, result7, expect_app_info)

        #
        # Delete entry (should succeed)
        # (test purposes only to keep the DB clean)
        #
        # result8 = ccv_01.delete()
        # self.assert_delete_success(MODULE_NAME, result8)

        #
        # Delete entry (should throw an error)
        # (test purposes only to keep the DB clean)
        #
        # result9 = ccv_01.delete()
        # self.assert_delete_error(MODULE_NAME, result9, RESOURCE_NOT_FOUND)

    def test_create_calibration_constant_version_from_dict(self):
        #
        # Create new entry (should succeed)
        #
        result1 = CalibrationConstantVersion.create_from_dict(self.cal_client,
                                                              self.ccv_01)
        self.assert_create_success(MODULE_NAME, result1, self.ccv_01)

        calibration_constant_version = result1['data']
        ccv_id = calibration_constant_version['id']
        ccv_name = calibration_constant_version['name']

        #
        # Create duplicated entry (should throw an error)
        #
        result2 = CalibrationConstantVersion.create_from_dict(self.cal_client,
                                                              self.ccv_01)
        expect_app_info = {'name': ['has already been taken']}
        self.assert_create_error(MODULE_NAME, result2, expect_app_info)

        #
        # Get entry by name
        #
        result3 = CalibrationConstantVersion.get_by_name(self.cal_client,
                                                         ccv_name)
        self.assert_find_success(MODULE_NAME, result3, self.ccv_01)

        #
        # Get entry by ID
        #
        result4 = CalibrationConstantVersion.get_by_id(self.cal_client, ccv_id)
        self.assert_find_success(MODULE_NAME, result4, self.ccv_01)

    def test_calibration_constants_get_closest_version_now(self):
        #
        # Search for the closest version
        #
        calibration_constant_ids = [-1, -2]
        physical_detector_unit_id = -1
        event_at = None
        snapshot_at = None

        result1 = CalibrationConstantVersion.get_closest_by_time(
            self.cal_client, calibration_constant_ids,
            physical_detector_unit_id,
            event_at, snapshot_at)

        # result1 ==>
        # {'success': True,
        #  'info': 'Got calibration_constant_version successfully',
        #  'app_info': {},
        #  'data': {
        #       'id': -4,
        #       'name': 'CALIBRATION_CONSTANT_VERSION_TEST-4_DO_NOT_DELETE',
        #       'file_name': 'cal.r0001.c0002.h5',
        #       'path_to_file': '/usr/local/cal_repo/',
        #       'data_set_name': 'exp001.cal001',
        #       'flg_deployed': True,
        #       'flg_good_quality': True,
        #       'begin_validity_at': '2015-05-25T10:30:14.000+02:00',
        #       'end_validity_at': '2029-06-24T00:00:00.000+02:00',
        #       'begin_at': '2015-05-26T12:00:00.000+02:00',
        #       'start_idx': None,
        #       'end_idx': None,
        #       'raw_data_location': None,
        #       'description': None,
        #       'calibration_constant': {
        #           'id': -2,
        #           'name': 'CALIBRATION_CONSTANT_TEST-2_DO_NOT_DELETE',
        #           'flg_auto_approve': True,
        #           'flg_available': True,
        #           'description': 'Created automatically via seed:seed_tests',
        #           'detector_type_id': -1,
        #           'calibration_id': -1,
        #           'condition_id': -2},
        #       'physical_detector_unit': {
        #           'id': -1,
        #           'name': 'PHYSICAL_DETECTOR_UNIT-1_DO_NOT_DELETE',
        #           'detector_type_id': -1,
        #           'flg_available': True,
        #           'description': 'Created automatically via seed:seed_tests'}
        # }}

        self.assert_find_success(MODULE_NAME, result1, self.expected_db_04)

    def test_calibration_constants_get_closest_version_in_the_past(self):
        #
        # Search for the closest version
        #
        calibration_constant_ids = [-1, -2]
        physical_detector_unit_id = -1
        event_at = '2014-05-24T12:00:00.000+02:00'
        snapshot_at = None

        result1 = CalibrationConstantVersion.get_closest_by_time(
            self.cal_client, calibration_constant_ids,
            physical_detector_unit_id,
            event_at, snapshot_at)

        # result1 ==>
        # {'success': True,
        #  'info': 'Got calibration_constant_version successfully',
        #  'app_info': {},
        #  'data': {
        #       'id': -2,
        #       'name': 'CALIBRATION_CONSTANT_VERSION_TEST-2_DO_NOT_DELETE',
        #       'file_name': 'cal.r0001.c0002.h5',
        #       'path_to_file': '/usr/local/cal_repo/',
        #       'data_set_name': 'exp001.cal001',
        #       'flg_deployed': True,
        #       'flg_good_quality': True,
        #       'begin_validity_at': '2014-05-25T10:30:14.000+02:00',
        #       'end_validity_at': '2029-06-24T00:00:00.000+02:00',
        #       'begin_at': '2014-05-26T12:00:00.000+02:00',
        #       'start_idx': None,
        #       'end_idx': None,
        #       'raw_data_location': None,
        #       'description': None,
        #       'calibration_constant': {
        #           'id': -1,
        #           'name': 'CALIBRATION_CONSTANT_TEST-1_DO_NOT_DELETE',
        #           'flg_auto_approve': True,
        #           'flg_available': True,
        #           'description': 'Created automatically via seed:seed_tests',
        #           'detector_type_id': -1,
        #           'calibration_id': -1,
        #           'condition_id': -1},
        #       'physical_detector_unit': {
        #           'id': -1,
        #           'name': 'PHYSICAL_DETECTOR_UNIT-1_DO_NOT_DELETE',
        #           'detector_type_id': -1,
        #           'flg_available': True,
        #           'description': 'Created automatically via seed:seed_tests'}
        # }}

        self.assert_find_success(MODULE_NAME, result1, self.expected_db_02)

    def test_calibration_constants_get_prior_in_time(self):
        #
        # Search for the valid version among many
        #
        calibration_constant_ids = [-1, -2, -3, -4]
        physical_detector_unit_id = -1
        event_at = '2014-05-26T09:00:00.000+02:00'
        snapshot_at = None

        result1 = CalibrationConstantVersion.get_prior_in_time(
            self.cal_client, calibration_constant_ids,
            physical_detector_unit_id,
            event_at, snapshot_at)

        self.assert_find_success(MODULE_NAME, result1, self.expected_db_01)

    def test_calibration_constants_get_all_versions_now(self):
        #
        # Search for all the version
        #
        calibration_constant_ids = [-1, -2]
        physical_detector_unit_id = -1
        event_at = None
        snapshot_at = None

        result1 = CalibrationConstantVersion.get_all_versions(
            self.cal_client, calibration_constant_ids,
            physical_detector_unit_id,
            event_at, snapshot_at)

        assert [(d['id'], d['name']) for d in result1['data']] == [
            (-4, self.expected_db_04['name']),
            (-3, 'CALIBRATION_CONSTANT_VERSION_TEST-3_DO_NOT_DELETE'),
            (-2, self.expected_db_02['name']),
            (-1, 'CALIBRATION_CONSTANT_VERSION_TEST-1_DO_NOT_DELETE'),
        ]

    def test_calibration_constants_get_all_versions_in_the_past(self):
        #
        # Search for all the version
        #
        calibration_constant_ids = [-1, -2]
        physical_detector_unit_id = -1
        event_at = '2014-05-26T12:00:00.000+02:00'
        snapshot_at = None

        result1 = CalibrationConstantVersion.get_all_versions(
            self.cal_client, calibration_constant_ids,
            physical_detector_unit_id,
            event_at, snapshot_at)

        assert [(d['id'], d['name']) for d in result1['data']] == [
            (-2, self.expected_db_02['name']),
            (-1, 'CALIBRATION_CONSTANT_VERSION_TEST-1_DO_NOT_DELETE'),
            (-3, 'CALIBRATION_CONSTANT_VERSION_TEST-3_DO_NOT_DELETE'),
            (-4, self.expected_db_04['name']),
        ]

    def test_calibration_constant_versions_by_detector_conditions(self):
        calibration_ids = [-1, -2]
        detector_identifier = 'TEST_DET_CI-2'
        condition = {
            "parameters_conditions_attributes": [{"parameter_id": "-1",
                                                  "value": "2.5"},
                                                 {"parameter_id": "-2",
                                                  "value": "1.5"}]}

        results = CalibrationConstantVersion.get_closest_by_time_by_detector_conditions(  # noqa
            self.cal_client, detector_identifier,
            calibration_ids,
            condition)

        # sort results
        data_sorted = sorted(results['data'], key=lambda d: -d['id'])

        assert [(d['id'], d['name'],
                 d['physical_detector_unit']['physical_name']) for d in
                data_sorted] == [
                   (-7, 'CALIBRATION_CONSTANT_VERSION_TEST-7_DO_NOT_DELETE',
                    'PHYSICAL_DETECTOR_UNIT-1_DO_NOT_DELETE'),
                   (-8, 'CALIBRATION_CONSTANT_VERSION_TEST-8_DO_NOT_DELETE',
                    'PHYSICAL_DETECTOR_UNIT-2_DO_NOT_DELETE'),
                   (-9, 'CALIBRATION_CONSTANT_VERSION_TEST-9_DO_NOT_DELETE',
                    'PHYSICAL_DETECTOR_UNIT-3_DO_NOT_DELETE'),
                   (-10, 'CALIBRATION_CONSTANT_VERSION_TEST-10_DO_NOT_DELETE',
                    'PHYSICAL_DETECTOR_UNIT-1_DO_NOT_DELETE'),
                   (-11, 'CALIBRATION_CONSTANT_VERSION_TEST-11_DO_NOT_DELETE',
                    'PHYSICAL_DETECTOR_UNIT-2_DO_NOT_DELETE'),
                   (-12, 'CALIBRATION_CONSTANT_VERSION_TEST-12_DO_NOT_DELETE',
                    'PHYSICAL_DETECTOR_UNIT-3_DO_NOT_DELETE')
               ]

    #
    # fields_validation
    #

    def fields_validation(self, receive, expect):
        assert receive['name'] == expect['name']
        assert receive['file_name'] == expect['file_name']
        assert receive['path_to_file'] == expect['path_to_file']
        assert receive['data_set_name'] == expect['data_set_name']
        assert receive['flg_deployed'] == expect['flg_deployed']
        assert receive['flg_good_quality'] == expect['flg_good_quality']

        # datetime fields
        assert parse_dt(receive['begin_at']) == parse_dt(expect['begin_at'])
        assert parse_dt(receive['begin_validity_at']) \
               == parse_dt(expect['begin_validity_at'])
        assert parse_dt(receive['end_validity_at']) \
               == parse_dt(expect['end_validity_at'])

        assert receive['start_idx'] == expect['start_idx']
        assert receive['end_idx'] == expect['end_idx']
        assert receive['raw_data_location'] == expect['raw_data_location']
        assert receive['description'] == expect['description']

        receive_cc_id = receive['calibration_constant']['id']
        assert receive_cc_id == expect['calibration_constant_id']

        receive_cc_id = receive[PHYSICAL_DETECTOR_UNIT]['id']
        assert receive_cc_id == expect['physical_detector_unit_id']


if __name__ == '__main__':
    unittest.main()



"""ParameterTest class"""

import unittest

import pytest

from .module_base import ModuleBase
from ..common.config_test import PARAMETER, RESOURCE_NOT_FOUND
from ..common.generators import generate_unique_name
from ...modules.parameter import Parameter

MODULE_NAME = PARAMETER


@pytest.mark.usefixtures('client_cls')
class ParameterTest(ModuleBase, unittest.TestCase):
    def setUp(self):
        __unique_name1 = generate_unique_name('Parameter01')
        self.par_01 = {
            'name': __unique_name1,
            'unit_id': -1,
            'flg_available': True,
            'flg_logarithmic': False,
            'def_lower_deviation_value': 1.0,
            'def_upper_deviation_value': 1.2,
            'description': 'desc 01'
        }

        __unique_name_upd = generate_unique_name('ParameterUpd01')
        self.par_01_upd = {
            'name': __unique_name1,
            'unit_id': -1,
            'flg_available': True,
            'flg_logarithmic': False,
            'def_lower_deviation_value': 1.0,
            'def_upper_deviation_value': 1.2,
            'description': 'desc 01'
        }

    def test_create_parameter(self):
        param_01 = Parameter(
            calibration_client=self.cal_client,
            name=self.par_01['name'],
            unit_id=self.par_01['unit_id'],
            flg_available=self.par_01['flg_available'],
            flg_logarithmic=self.par_01['flg_logarithmic'],
            def_lower_deviation_value=self.par_01['def_lower_deviation_value'],
            def_upper_deviation_value=self.par_01['def_upper_deviation_value'],
            description=self.par_01['description'])

        # Create new entry (should succeed)
        result1 = param_01.create()
        self.assert_create_success(MODULE_NAME, result1, self.par_01)

        parameter_id = result1['data']['id']
        parameter_name = result1['data']['name']

        try:
            # Create duplicated entry (should throw an error)
            result2 = param_01.create()
            expect_app_info = {'name': ['has already been taken']}
            self.assert_create_error(MODULE_NAME, result2, expect_app_info)

            # Get entry by name
            result3 = Parameter.get_by_name(self.cal_client, parameter_name)
            self.assert_find_success(MODULE_NAME, result3, self.par_01)

            # Get entry by ID
            result4 = Parameter.get_by_id(self.cal_client, parameter_id)
            self.assert_find_success(MODULE_NAME, result4, self.par_01)

            # Get entry with non-existent ID (should throw an error)
            parameter_id = -666
            result5 = Parameter.get_by_id(self.cal_client, parameter_id)
            self.assert_find_error(MODULE_NAME, result5, RESOURCE_NOT_FOUND)

            # Put entry information (update some fields should succeed)
            param_01.name = self.par_01_upd['name']
            param_01.flg_available = self.par_01_upd['flg_available']
            param_01.description = self.par_01_upd['description']
            result6 = param_01.update()
            self.assert_update_success(MODULE_NAME, result6, self.par_01_upd)

            # Put entry information (update some fields should throw an error)
            param_01.name = '__THIS_NAME_CANNOT_BE_UPDATED'  # noqa
            param_01.flg_available = self.par_01_upd['flg_available']
            param_01.description = self.par_01_upd['description']
            result7 = param_01.update()
            expect_app_info = {
                'name': ['cannot be updated']}
            self.assert_update_error(MODULE_NAME, result7, expect_app_info)

        finally:
            # Delete entry (should succeed)
            # (test purposes only to keep the DB clean)
            result8 = param_01.delete()

        # Check deletion worked if there isn't already an error to report
        self.assert_delete_success(MODULE_NAME, result8)

        # Delete entry (should throw an error)
        # (test purposes only to keep the DB clean)
        result9 = param_01.delete()
        self.assert_delete_error(MODULE_NAME, result9, RESOURCE_NOT_FOUND)

    def fields_validation(self, receive, expect):
        assert receive['name'] == expect['name']
        assert receive['unit_id'] == expect['unit_id']
        assert receive['flg_available'] == expect['flg_available']
        assert receive['flg_logarithmic'] == expect['flg_logarithmic']
        assert receive['def_lower_deviation_value'] \
               == expect['def_lower_deviation_value']
        assert receive['def_upper_deviation_value'] \
               == expect['def_upper_deviation_value']
        assert receive['description'] == expect['description']


if __name__ == '__main__':
    unittest.main()



"""CalibrationConstantVersionApiTest class"""

import unittest
from http import HTTPStatus

import pytest
from dateutil.parser import parse as parse_dt

from .api_base import ApiBase
from ..common import generators
from ..common.config_test import *


@pytest.mark.usefixtures('client_cls')
class CalibrationConstantVersionApiTest(ApiBase, unittest.TestCase):
    __unique_name = generators.generate_unique_name('CCVersionApi')
    __unique_file_name = generators.generate_unique_file_name()
    __begin_at = generators.generate_timestamp_str(0)

    __calibration_constant_version_01 = {
        'calibration_constant_version': {
            'name': __unique_name,
            'file_name': __unique_file_name,
            'path_to_file': 'path_to_file_1',
            'data_set_name': 'data_set_name_1',
            'calibration_constant_id': -3,
            'physical_detector_unit_id': -1,
            'flg_deployed': True,
            'flg_good_quality': True,
            'begin_validity_at': '2014-05-25T08:30:00.000+02:00',
            'end_validity_at': '2025-12-25T08:30:00.000+01:00',
            'begin_at': __begin_at,
            'start_idx': 1,
            'end_idx': 2,
            'raw_data_location': 'XFEL/metadata/',
            'description': 'desc version 01'
        }
    }

    def test_create_calibration_constant_version_api(self):
        ccv = self.__calibration_constant_version_01
        expect = ccv['calibration_constant_version']

        # Create new entry (should succeed)
        received = self.__create_entry_api(ccv, expect)

        ccv_id = received['id']
        ccv_name = received['name']

        # Create duplicated entry (should throw an error)
        self.__create_error_entry_uk_api(ccv)

        # event_at == expected_hash['begin_at']
        # This is necessary, because the CalibrationConstantVersionApi.begin_at
        # current_DATETIME + Random(60 seconds) is part of the Unique Key that
        # identifies uniquely a Calibration Constant Version entry
        self.__test_get_version_int(expect['begin_at'],
                                    expect_ccv_id=ccv_id)

        # Get entry by name
        self.__get_all_entries_by_name_api(ccv_name, expect)

        # Get entry by ID
        self.__get_entry_by_id_api(ccv_id, expect)

        # Put entry information (update some fields should succeed)
        self.__update_entry_api(ccv_id, expect)

    def fields_validation(self, receive, expect):
        assert receive['name'] == expect['name']
        assert receive['file_name'] == expect['file_name']
        assert receive['path_to_file'] == expect['path_to_file']
        assert receive['data_set_name'] == expect['data_set_name']
        assert receive['flg_deployed'] == expect['flg_deployed']
        assert receive['flg_good_quality'] == expect['flg_good_quality']
        assert parse_dt(receive['begin_at']) == parse_dt(expect['begin_at'])
        assert parse_dt(receive['begin_validity_at']) \
               == parse_dt(expect['begin_validity_at'])
        assert parse_dt(receive['end_validity_at']) \
               == parse_dt(expect['end_validity_at'])

        receive_cc_id = receive['calibration_constant']['id']
        assert receive_cc_id == expect['calibration_constant_id']

        receive_cc_id = receive[PHYSICAL_DETECTOR_UNIT]['id']
        assert receive_cc_id == expect['physical_detector_unit_id']

    def __create_entry_api(self, entry_info, expect):
        response = self.cal_client.create_calibration_constant_version_api(
            entry_info)
        receive = self.get_and_validate_create_entry(response)
        self.fields_validation(receive, expect)
        return receive

    def __create_error_entry_uk_api(self, entry_info):
        response = self.cal_client.create_calibration_constant_version_api(
            entry_info)
        receive = self.load_response_content(response)

        expect = {'info': {'name': ['has already been taken'],
                           'calibration_constant_id': [
                               'has already been taken'
                           ],
                           'physical_detector_unit_id': [
                               'has already been taken'],
                           'begin_at': ['has already been taken']}}

        assert receive == expect
        assert response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY

    def __test_get_version_int(self, event_at, expect_ccv_id=None):
        ccv = self.__calibration_constant_version_01
        expect = ccv['calibration_constant_version']

        calibration_constant_id = expect['calibration_constant_id']
        physical_detector_unit_id = expect['physical_detector_unit_id']
        # event_at = '' #'2014-10-13T09:13:26.000+02:00'
        snapshot_at = ''

        resp = self.cal_client.get_calibration_constant_version_by_uk_api(
            calibration_constant_id, physical_detector_unit_id, event_at,
            snapshot_at)
        receive = self.load_response_content(resp)

        self.fields_validation(receive, expect)
        assert resp.status_code == HTTPStatus.OK

        if expect_ccv_id is not None:
            assert receive['id'] == expect_ccv_id

    def __get_all_entries_by_name_api(self, name, expect, page=DEF_PAGE,
                                      page_size=DEF_PAGE_SIZE):
        resp = self.cal_client. \
            get_all_calibration_constant_versions_by_name_api(name, page, page_size)

        receive = self.get_and_validate_all_entries_by_name(resp)
        self.fields_validation(receive, expect)

    def __get_entry_by_id_api(self, entry_id, expect):
        resp = self.cal_client.get_calibration_constant_version_by_id_api(
            entry_id)
        receive = self.get_and_validate_entry_by_id(resp)
        self.fields_validation(receive, expect)

    def __update_entry_api(self, entry_id, expect):
        __unique_name = generators.generate_unique_name('CCVersionApiUpd')
        __unique_file_name = generators.generate_unique_file_name()
        __begin_at = generators.generate_timestamp_str(1)
        ccv_upd = {
            'calibration_constant_version': {
                'name': __unique_name,
                'file_name': __unique_file_name,
                'path_to_file': 'path_to_file_2',
                'data_set_name': 'data_set_name_2',
                # 'calibration_constant_id': '-1',
                # 'physical_detector_unit_id': '-1',
                'flg_deployed': False,
                'flg_good_quality': False,
                'begin_validity_at': '2014-05-25T08:31:00.000+02:00',
                'end_validity_at': '2025-12-25T08:31:00.000+01:00',
                'begin_at': __begin_at,
                'start_idx': 10,
                'end_idx': 20,
                'raw_data_location': 'XFEL/metadata_bck/',
                'description': 'desc version 01 UPDATED!'
            }
        }

        response = self.cal_client.update_calibration_constant_version_api(
            entry_id,
            ccv_upd)

        receive = self.load_response_content(response)

        # Add parameters not send to the update API
        expect_upd = ccv_upd['calibration_constant_version']
        expect_upd['calibration_constant_id'] = -3
        expect_upd['physical_detector_unit_id'] = -1

        self.fields_validation(receive, expect_upd)
        assert response.status_code == HTTPStatus.OK

        assert expect['name'] != expect_upd['name']
        assert expect['file_name'] != expect_upd['file_name']
        assert expect['path_to_file'] != expect_upd['path_to_file']
        assert expect['data_set_name'] != expect_upd['data_set_name']
        assert expect['flg_deployed'] != expect_upd['flg_deployed']
        assert expect['flg_good_quality'] != expect_upd['flg_good_quality']
        assert expect['begin_validity_at'] != expect_upd['begin_validity_at']
        assert expect['end_validity_at'] != expect_upd['end_validity_at']
        assert expect['begin_at'] != expect_upd['begin_at']
        assert expect['start_idx'] != expect_upd['start_idx']
        assert expect['end_idx'] != expect_upd['end_idx']
        assert expect['raw_data_location'] != expect_upd['raw_data_location']
        assert expect['description'] != expect_upd['description']


if __name__ == '__main__':
    unittest.main()



"""UserApi module class"""

from ..common.base import Base


class UserApi(Base):
    def get_current_user(self):
        api_url = self.__get_api_url(model_name='me')
        return self.api_get(api_url)

    #
    # Private Helper Methods
    #
    def __get_api_url(self, model_name='user', api_specifics=''):
        return self.get_api_url(model_name, api_specifics)



"""CalibrationApiTest class"""

import unittest
from http import HTTPStatus

import pytest

from .api_base import ApiBase
from ..common.generators import generate_unique_name
from ..common.config_test import *


@pytest.mark.usefixtures('client_cls')
class CalibrationApiTest(ApiBase, unittest.TestCase):
    def test_create_calibration_api(self):
        __unique_name = generate_unique_name('CalibrationApi')
        calibration = {
            'calibration': {
                'name': __unique_name,
                'unit_id': -1,
                'max_value': 10.0,
                'min_value': 1.0,
                'allowed_deviation': 0.1,
                'description': 'desc 01'
            }
        }

        expect = calibration['calibration']

        #
        # Create new entry (should succeed)
        #
        received = self.__create_entry_api(calibration, expect)

        calibration_id = received['id']
        calibration_name = received['name']

        try:
            # Create duplicated entry (should throw an error)
            self.__create_error_entry_uk_api(calibration)

            # Get entry by name
            self.__get_all_entries_by_name_api(calibration_name, expect)

            # Get entry by ID
            self.__get_entry_by_id_api(calibration_id, expect)

            # Put entry information (update some fields should succeed)
            self.__update_entry_api(calibration_id, expect)

        finally:
            # Delete entry (should succeed)
            # (test purposes only to keep the DB clean)
            del_resp = self.cal_client.delete_calibration_api(calibration_id)

        # Validate the response from deletion if nothing already error-ed
        self.get_and_validate_delete_entry_by_id(del_resp)

    def fields_validation(self, receive, expect):
        assert receive['name'] == expect['name']
        assert receive['unit_id'] == expect['unit_id']
        assert receive['max_value'] == expect['max_value']
        assert receive['min_value'] == expect['min_value']
        assert receive['allowed_deviation'] == expect['allowed_deviation']
        assert receive['description'] == expect['description']

    #
    # Internal private APIs methods
    #
    def __create_entry_api(self, entry_info, expect):
        response = self.cal_client.create_calibration_api(entry_info)
        receive = self.get_and_validate_create_entry(response)
        self.fields_validation(receive, expect)
        return receive

    def __create_error_entry_uk_api(self, entry_info):
        response = self.cal_client.create_calibration_api(entry_info)
        receive = self.load_response_content(response)

        expect = {'info': {'name': ['has already been taken'],
                           'unit': ['has already been taken']}}

        assert receive == expect
        assert response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY

    def __update_entry_api(self, entry_id, expect):
        unique_name = generate_unique_name('CalibrationApiUpd')
        calibration_upd = {
            'calibration': {
                'name': expect['name'],
                # 'unit_id': '-1',
                'max_value': 12.0,
                'min_value': 3.0,
                'allowed_deviation': 0.5,
                'description': 'desc 01 updated!'
            }
        }

        resp = self.cal_client.update_calibration_api(entry_id,
                                                      calibration_upd)
        receive = self.load_response_content(resp)

        # Add parameters not send to the update API
        calibration_upd['calibration']['unit_id'] = -1
        expect_upd = calibration_upd['calibration']

        self.fields_validation(receive, expect_upd)
        assert resp.status_code == HTTPStatus.OK

        assert expect['max_value'] != expect_upd['max_value']
        assert expect['min_value'] != expect_upd['min_value']
        assert expect['allowed_deviation'] != expect_upd['allowed_deviation']
        assert expect['description'] != expect_upd['description']

    def __get_all_entries_by_name_api(self, name, expect,
                                      page=DEF_PAGE, page_size=DEF_PAGE_SIZE):
        response = self.cal_client.get_all_calibrations_by_name_api(name, page, page_size)
        receive = self.get_and_validate_all_entries_by_name(response)
        self.fields_validation(receive, expect)

    def __get_entry_by_id_api(self, entry_id, expect):
        response = self.cal_client.get_calibration_by_id_api(entry_id)
        receive = self.get_and_validate_entry_by_id(response)
        self.fields_validation(receive, expect)


if __name__ == '__main__':
    unittest.main()



"""CalibrationClientApi class"""

from warnings import warn

from .calibration_client import CalibrationClient


class CalibrationClientApi(CalibrationClient):
    """DEPRECATED: use CalibrationClient instead"""

    def __init__(self, *args, **kwargs):
        warn(
            "CalibrationClientApi is deprecated, please use CalibrationClient",
            stacklevel=2,
        )
        super().__init__(*args, **kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
from os import path

from setuptools import setup, find_packages

# read the contents of your README file
package_dir = path.abspath(path.dirname(__file__))
with open(path.join(package_dir, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()


def find_version():
    filename = path.join(package_dir, 'calibration_client', '__init__.py')
    with open(filename, 'r', encoding='utf-8') as f:
        vers_file = f.read()
    print('#' * 100)
    print('filename => {0}'.format(filename))
    print('file content => {0}'.format(vers_file.splitlines()[:10]))
    print('#' * 100)
    match = re.search(r"^__version__ = '([.\d]+)'", vers_file, re.M)
    if match is not None:
        return match.group(1)
    raise RuntimeError("Unable to find version string.")


setup(
    name='calibration_client',
    version=find_version(),
    description='Python Client for European XFEL Calibration Catalogue Web App'
                ' available at https://in.xfel.eu/calibration',
    long_description=long_description,
    author='Luís Maia',
    author_email='luis.maia@xfel.eu',
    maintainer='Luís Maia',
    maintainer_email='luis.maia@xfel.eu',
    url='https://git.xfel.eu/gitlab/ITDM/calibration_client',
    platforms='any',
    license='MIT',
    packages=find_packages(),
    install_requires=['oauthlib',
                      'requests',
                      'requests-oauthlib',
                      'oauth2_xfel_client >=6.1',
                      'pytz'],
    extras_require={'test': [
        'pytest',
        'pytest-cov',
        'python-dateutil',
        'pytz',
        'pycodestyle'
    ]},
    python_requires='>=3.7',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)



"""CalibrationTest class"""

import unittest

import pytest

from .module_base import ModuleBase
from ..common.config_test import CALIBRATION, RESOURCE_NOT_FOUND
from ..common.generators import generate_unique_name
from ...modules.calibration import Calibration

MODULE_NAME = CALIBRATION


@pytest.mark.usefixtures('client_cls')
class CalibrationTest(ModuleBase, unittest.TestCase):
    def setUp(self):
        __unique_name1 = generate_unique_name('Calibration01')
        self.cal_01 = {
            'name': __unique_name1,
            'unit_id': -1,
            'max_value': 10.0,
            'min_value': 1.0,
            'allowed_deviation': 0.1,
            'description': 'desc 01'
        }

        __unique_name_upd = generate_unique_name('CalibrationUpd01')
        self.cal_01_upd = {
            'name': __unique_name1,
            'unit_id': -1,
            'max_value': 12.0,
            'min_value': 3.0,
            'allowed_deviation': 0.5,
            'description': 'desc 01 updated!'
        }

    def test_create_calibration(self):
        cal_01 = Calibration(
            calibration_client=self.cal_client,
            name=self.cal_01['name'],
            unit_id=self.cal_01['unit_id'],
            max_value=self.cal_01['max_value'],
            min_value=self.cal_01['min_value'],
            allowed_deviation=self.cal_01['allowed_deviation'],
            description=self.cal_01['description']
        )

        # Create new entry (should succeed)
        result1 = cal_01.create()
        self.assert_create_success(MODULE_NAME, result1, self.cal_01)

        calibration = result1['data']
        calibration_id = result1['data']['id']
        calibration_name = result1['data']['name']

        try:
            # Create duplicated entry (should throw an error)
            cal_01_dup = cal_01
            result2 = cal_01_dup.create()
            expect_app_info = {'name': ['has already been taken'],
                               'unit': ['has already been taken']}
            self.assert_create_error(MODULE_NAME, result2, expect_app_info)

            # Get entry by name
            result3 = Calibration.get_by_name(self.cal_client,
                                              calibration_name)
            self.assert_find_success(MODULE_NAME, result3, self.cal_01)

            # Get entry by ID
            result4 = Calibration.get_by_id(self.cal_client, calibration_id)
            self.assert_find_success(MODULE_NAME, result4, self.cal_01)

            # Get entry with non-existent ID (should throw an error)
            calibration_id = -666
            result5 = Calibration.get_by_id(self.cal_client, calibration_id)
            self.assert_find_error(MODULE_NAME, result5, RESOURCE_NOT_FOUND)

            # Put entry information (update some fields should succeed)
            cal_01.name = self.cal_01_upd['name']
            cal_01.max_value = self.cal_01_upd['max_value']
            cal_01.min_value = self.cal_01_upd['min_value']
            cal_01.allowed_deviation = self.cal_01_upd['allowed_deviation']
            cal_01.description = self.cal_01_upd['description']
            result6 = cal_01.update()
            self.assert_update_success(MODULE_NAME, result6, self.cal_01_upd)

            # Put entry information (update some fields should throw an error)
            cal_01.name = '__THIS_NAME_CANNOT_BE_UPDATED'  # noqa
            cal_01.max_value = self.cal_01_upd['max_value']
            cal_01.min_value = self.cal_01_upd['min_value']
            cal_01.allowed_deviation = self.cal_01_upd['allowed_deviation']
            cal_01.description = self.cal_01_upd['description']
            result7 = cal_01.update()
            expect_app_info = {
                'name': ['cannot be updated']}
            self.assert_update_error(MODULE_NAME, result7, expect_app_info)

        finally:
            # Delete entry (should succeed)
            # (test purposes only to keep the DB clean)
            result8 = cal_01.delete()

        # Check deletion worked if there isn't already an error to report
        self.assert_delete_success(MODULE_NAME, result8)

        # Delete entry (should throw an error)
        # (test purposes only to keep the DB clean)
        result9 = cal_01.delete()
        self.assert_delete_error(MODULE_NAME, result9, RESOURCE_NOT_FOUND)

    def fields_validation(self, receive, expect):
        assert receive['name'] == expect['name']
        assert receive['unit_id'] == expect['unit_id']
        assert receive['max_value'] == expect['max_value']
        assert receive['min_value'] == expect['min_value']
        assert receive['allowed_deviation'] == expect['allowed_deviation']
        assert receive['description'] == expect['description']


if __name__ == '__main__':
    unittest.main()



"""BaseApiTest Class with helper methods common to all modules tests"""

import unittest
from http import HTTPStatus

from calibration_client.common.base import Base
from ..common.config_test import RESOURCE_NOT_FOUND


class ApiBase(unittest.TestCase):
    @staticmethod
    def load_response_content(response):
        return Base.load_json_from_content(response)

    def get_and_validate_create_entry(self, response):
        assert response.status_code == HTTPStatus.CREATED

        return self.load_response_content(response)

    def get_and_validate_set_entry(self, response):
        assert response.status_code in [HTTPStatus.CREATED, HTTPStatus.OK]

        return self.load_response_content(response)

    def get_and_validate_all_entries_by_name(self, response):
        assert response.status_code == HTTPStatus.OK

        resp_content = self.load_response_content(response)
        return resp_content[0]

    def get_and_validate_entry_by_id(self, response):
        assert response.status_code == HTTPStatus.OK

        return self.load_response_content(response)

    def get_and_validate_delete_entry_by_id(self, response):
        assert response.status_code == HTTPStatus.NO_CONTENT

        assert self.load_response_content(response) == {}

    def get_and_validate_resource_not_found(self, response):
        receive = self.load_response_content(response)
        expect = {'info': RESOURCE_NOT_FOUND}

        assert response.status_code == HTTPStatus.NOT_FOUND
        assert receive == expect



"""ConditionTest class"""

import unittest

import pytest

from .module_base import ModuleBase
from ..common.config_test import CONDITION, RESOURCE_NOT_FOUND
from ..common.generators import generate_unique_name
from ...modules.condition import Condition

MODULE_NAME = CONDITION


@pytest.mark.usefixtures('client_cls')
class ConditionTest(ModuleBase, unittest.TestCase):
    def setUp(self):
        __parameters_conditions_attr_01 = [
            {
                'parameter_id': -1,
                'value': 2.5,
                'lower_deviation_value': 0.5,
                'upper_deviation_value': 0.5,
                'flg_available': True,
                'description': 'Created automatically via seed:seed_tests'
            },
            {
                'parameter_id': -2,
                'value': 1.5,
                'lower_deviation_value': 0.5,
                'upper_deviation_value': 0.5,
                'flg_available': True,
                'description': 'Created automatically via seed:seed_tests'
            }
        ]

        __parameters_conditions_attr_02 = [
            {
                'parameter_id': -1,
                'value': 2.4,
                'lower_deviation_value': 0.5,
                'upper_deviation_value': 0.5,
                'flg_available': True,
                'description': 'Created automatically via seed:seed_tests'
            },
            {
                'parameter_id': -2,
                'value': 1.3,
                'lower_deviation_value': 0.5,
                'upper_deviation_value': 0.5,
                'flg_available': True,
                'description': 'Created automatically via seed:seed_tests'
            }
        ]

        __parameters_conditions_attr_03 = [
            {
                'parameter_id': -1,
                'value': 2.7,
                'lower_deviation_value': 0.5,
                'upper_deviation_value': 0.5,
                'flg_available': True,
                'description': 'Created automatically via seed:seed_tests'
            },
            {
                'parameter_id': -2,
                'value': 1.8,
                'lower_deviation_value': 0.5,
                'upper_deviation_value': 0.5,
                'flg_available': True,
                'description': 'Created automatically via seed:seed_tests'
            }
        ]

        __parameters_conditions_attr_02 = [
            {
                'parameter_id': -1,
                'value': 2.4,
                'lower_deviation_value': 0.5,
                'upper_deviation_value': 0.5,
                'flg_available': True,
                'description': 'Created automatically via seed:see_tests'
            },
            {
                'parameter_id': -2,
                'value': 1.3,
                'lower_deviation_value': 0.5,
                'upper_deviation_value': 0.5,
                'flg_available': True,
                'description': 'Created automatically via seed:see_tests'
            }
        ]

        __parameters_conditions_attr_03 = [
            {
                'parameter_id': -1,
                'value': 2.7,
                'lower_deviation_value': 0.5,
                'upper_deviation_value': 0.5,
                'flg_available': True,
                'description': 'Created automatically via seed:see_tests'
            },
            {
                'parameter_id': -2,
                'value': 1.8,
                'lower_deviation_value': 0.5,
                'upper_deviation_value': 0.5,
                'flg_available': True,
                'description': 'Created automatically via seed:see_tests'
            }
        ]

        __unique_name = 'CONDITION_TEST-1_DO_NOT_DELETE'

        self.cond_01 = {
            'name': __unique_name,
            'flg_available': True,
            'event_at': '2019-07-20 09:14:52',
            'description': 'Created automatically via seed:seed_tests',
            'parameters_conditions_attributes': __parameters_conditions_attr_01
        }

        self.cond_02 = {
            'name': 'CONDITION_TEST-2_DO_NOT_DELETE',
            'flg_available': True,
            'event_at': '2019-07-20 09:14:52',
            'description': 'Created automatically via seed:seed_tests',
            'parameters_conditions_attributes': __parameters_conditions_attr_02
        }

        self.cond_03 = {
            'name': 'CONDITION_TEST-3_DO_NOT_DELETE',
            'flg_available': True,
            'event_at': '2019-07-20 09:14:52',
            'description': 'Created automatically via seed:seed_tests',
            'parameters_conditions_attributes': __parameters_conditions_attr_03
        }

        __unique_name_upd = generate_unique_name('ConditionUpd01')
        self.cond_01_upd = {
            'name': __unique_name,
            'flg_available': True,
            'event_at': '2019-07-20 09:14:52',
            'description': 'Created automatically via seed:seed_tests',
            'parameters_conditions_attributes': __parameters_conditions_attr_01
        }

    def test_create_condition(self):
        cond_01 = Condition(
            calibration_client=self.cal_client,
            name=self.cond_01['name'],
            flg_available=self.cond_01['flg_available'],
            event_at=None,
            parameters_conditions_attributes=self.cond_01[
                'parameters_conditions_attributes'],
            description=self.cond_01['description']
        )

        #
        # Set Condition (should succeed with existent)
        #
        result1 = cond_01.set_expected()
        self.assert_find_success(MODULE_NAME, result1, self.cond_01)

        condition = result1['data']
        condition_id = result1['data']['id']
        condition_name = result1['data']['name']

        #
        # Set duplicated Condition (should succeed with existent)
        #
        cond_01_dup = cond_01
        result2 = cond_01_dup.set_expected()
        self.assert_find_success(MODULE_NAME, result2, self.cond_01)

        #
        # Get expected
        #
        result3 = cond_01.get_expected()
        self.assert_find_success(MODULE_NAME, result3, self.cond_01)

        #
        # Get possible
        #
        res4 = cond_01.get_possible()
        possible_conditions_list = [self.cond_03, self.cond_02, self.cond_01]
        self.assert_find_success(MODULE_NAME, res4, possible_conditions_list)

        #
        # Get entry with non-existent Condition (should throw an error)
        #
        cond_01.parameters_conditions_attributes[0]['value'] = '200'

        result5 = cond_01.get_expected()
        self.assert_find_error(MODULE_NAME, result5, RESOURCE_NOT_FOUND)

    def test_create_condition_from_dict_success(self):
        __parameters_01 = [
            {
                'parameter_id': -1,
                'value': 2.5,
                'lower_deviation_value': 0.5,
                'upper_deviation_value': 0.5,
                'flg_available': True,
                'description': 'Created automatically via seed:seed_tests'
            },
            {
                'parameter_id': -2,
                'value': 1.5,
                'lower_deviation_value': 0.5,
                'upper_deviation_value': 0.5,
                'flg_available': True,
                'description': 'Created automatically via seed:seed_tests'
            }
        ]

        __unique_name = 'CONDITION_TEST-1_DO_NOT_DELETE'

        cond_dict = {
            'name': __unique_name,
            'flg_available': True,
            'description': 'Created automatically via seed:seed_tests',
            'parameters_conditions_attributes': __parameters_01
        }

        #
        # Set Condition from DICT (should succeed with existent)
        #
        resp = Condition.set_condition_from_dict(self.cal_client, cond_dict)

        expect = {'id': -1,
                  'flg_available': True,
                  'name': 'CONDITION_TEST-1_DO_NOT_DELETE',
                  'description': 'Created automatically via seed:seed_tests',
                  'num_parameters': 2}

        msg = 'Got {0} successfully'.format(MODULE_NAME)
        expect = {'success': True, 'info': msg, 'app_info': {}, 'data': expect}

        assert resp['app_info'] == {}
        assert resp['info'] == expect['info']
        assert resp['success'] == expect['success']

        # Validate all 'data' fields
        expect_data = expect['data']
        if type(expect_data) is dict and len(expect_data) > 0:
            for key, val in expect_data.items():
                assert resp['data'][key] == expect_data[key]
        else:
            assert resp['data'] == expect_data

    def test_create_condition_from_dict_error(self):
        __parameters_01 = [
            {
                'parameter_id': -1,
                'value': 100.5,
                'lower_deviation_value': 10.5,
                'upper_deviation_value': 10.5,
                'flg_available': 'True',
                'description': 'Created automatically via seed:seed_tests'
            }
        ]

        __unique_name = 'CONDITION_TEST-1_DO_NOT_DELETE'

        expect = {
            'name': __unique_name,
            'flg_available': True,
            'description': 'Created automatically via seed:seed_tests',
            'parameters_conditions_attributes': __parameters_01
        }

        #
        # Set Condition from DICT (should succeed with existent)
        #
        resp = Condition.set_condition_from_dict(self.cal_client, expect)

        expect_app_info = {'name': ['has already been taken']}
        self.assert_find_error(MODULE_NAME, resp, expect_app_info)

    #
    # fields_validation
    #
    def fields_validation(self, receive, expect):
        assert receive['name'] == expect['name']
        assert receive['flg_available'] == expect['flg_available']
        assert receive['description'] == expect['description']

        num_params = len(expect['parameters_conditions_attributes'])
        assert receive['num_parameters'] == num_params


if __name__ == '__main__':
    unittest.main()



"""ConditionApi module class"""

import json

from ..common.base import Base


class ConditionApi(Base):
    def get_possible_conditions(self, condition):
        api_url = self.__get_api_url('get_possible_conditions')

        return self.api_get(api_url, data=json.dumps(condition))

    def get_expected_condition(self, condition):
        api_url = self.__get_api_url('get_expected_condition')

        return self.api_get(api_url, data=json.dumps(condition))

    def set_expected_condition(self, condition):
        api_url = self.__get_api_url('set_expected_condition')

        return self.api_post(api_url, data=json.dumps(condition))

    #
    # Private Methods
    #
    def __get_api_url(self, api_specifics=''):
        model_name = 'conditions/'
        return self.get_api_url(model_name, api_specifics)



"""Condition module class"""

import logging
from datetime import datetime
from http import HTTPStatus

from ..common.base import Base
from ..common.config import *
from ..common.util import Util

MODULE_NAME = CONDITION


class Condition:
    def __init__(self, calibration_client, name, flg_available, event_at,
                 parameters_conditions_attributes, description=''):
        self.calibration_client = calibration_client
        self.id = None
        self.name = name
        self.flg_available = flg_available

        if event_at is None:
            self.event_at = datetime.today()
        else:
            self.event_at = event_at

        self.parameters_conditions_attributes = parameters_conditions_attributes  # noqa
        self.description = description

    def set_expected(self):
        cal_client = self.calibration_client
        response = cal_client.set_expected_condition(self.__get_resource())

        if response.status_code == HTTPStatus.CREATED:
            Base.cal_debug(MODULE_NAME, CREATE, response)
            res = Base.format_response(
                response, CREATE, HTTPStatus.CREATED, MODULE_NAME
            )

            if res['success']:
                self.id = res['data']['id']

        else:
            Base.cal_debug(MODULE_NAME, 'GET >> set_expected', response)
            res = Base.format_response(
                response, GET, HTTPStatus.OK, MODULE_NAME
            )

        return res

    # return the list of possible conditions order by the time distance
    # between the condition mesurement and the received event_at datetime
    def get_possible(self):
        cal_client = self.calibration_client
        response = cal_client.get_possible_conditions(self.__get_resource())

        Base.cal_debug(MODULE_NAME, 'get_possible', response)
        return Base.format_response(response, GET, HTTPStatus.OK, MODULE_NAME)

    def get_expected(self):
        cal_client = self.calibration_client
        response = cal_client.get_expected_condition(self.__get_resource())

        Base.cal_debug(MODULE_NAME, 'get_expected', response)
        return Base.format_response(response, GET, HTTPStatus.OK, MODULE_NAME)

    @staticmethod
    def set_condition_from_dict(cal_client, condition_h):
        new_cond = Condition(
            calibration_client=cal_client,
            name=condition_h['name'],
            flg_available=condition_h['flg_available'],
            event_at=None,
            parameters_conditions_attributes=condition_h[
                'parameters_conditions_attributes'],
            description=condition_h['description']
        )

        resp = new_cond.set_expected()
        logging.debug('Build condition successfully: {0}'.format(resp))

        return resp

    def __get_resource(self):
        condition = {
            CONDITION: {
                'name': self.name,
                'flg_available': self.flg_available,
                'event_at': Util.datetime_converter(self.event_at),
                'parameters_conditions_attributes':
                    self.parameters_conditions_attributes,
                'description': self.description
            }
        }

        return condition
